/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "previewitemcolor.h"

#include "themeattributeimage.h"

#include <qpainter.h>

#include <kstandarddirs.h> 


PreviewItemColor::PreviewItemColor(int left, int top, int width, int height, ThemeAttribute *attribute, int alpha, const QString &bitmapname, ThemeAttribute *depends_on)
 : PreviewItem(left, top)
{
  this->width = width;
  this->height = height;
  this->alpha = alpha;
  this->attribute = (ThemeAttributeColor *)attribute;
  this->depends_on = depends_on;
  connect(attribute, SIGNAL( signalValueChanged(const QString &) ), this, SIGNAL( signalValueChanged () ));
  connect(attribute, SIGNAL(signalEnterFocus()), this, SLOT(slotEnterFocus()));
  connect(attribute, SIGNAL(signalLostFocus()), this, SLOT(slotLostFocus()));
  if (!bitmapname.isNull()) {
    bitmap = QBitmap( locate("appdata", bitmapname) );
    text = QPixmap(width, height);
    text.setMask(bitmap);
  }
  else {
    bitmap = QBitmap();
  }
}


PreviewItemColor::~PreviewItemColor()
{
}

QString PreviewItemColor::getToolTip() {
  return attribute->get_description();
}

void PreviewItemColor::draw(QPainter* p)
{
  if ( depends_on != 0 && (! depends_on->get_value().isEmpty()) ) {
    //return;
  }
  else {
    QColor color = QColor("#"+attribute->get_value());
    // Qt 4.0: color.setAlpha ( 50 );
/*    QColor color = QColor("#"+apply_alpha(attribute->get_value(), alpha));*/
    if ( color.isValid() ) {
      if ( bitmap.isNull() ) {
        p->fillRect( left, top, width, height, color );
      }
      else {
        text.fill( color );
        p->drawPixmap (left, top, text) ;
      }
    }
  //}
  }
  if ( has_focus ) {
      QColor color;
      //if ( attribute->get_value().isEmpty() ) {
        color = Qt::red;
//       } else {
//         color = QColor("#"+attribute->get_value());
//       }
      p->setPen( color );
      p->drawRect( left, top, width, height );
  }
}

