/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "previewitemimage.h"

#include "animchecker.h"

#include <qpainter.h>
#include <qpaintdevice.h>
#include <qsize.h>
#include <qmovie.h>
#include <qbitmap.h>



PreviewItemImage::PreviewItemImage(int left, int top, int width, int height, ThemeAttribute *attribute, int leftoff, int topoff, bool clip)
 : QObject(), PreviewItem(left, top)
{
  this->width = width;
  this->height = height;
  this->attribute = (ThemeAttributeImage *)attribute;
  connect(attribute, SIGNAL( signalValueChanged(const QString &) ), this, SIGNAL( signalValueChanged () ));
  //connect(attribute, SIGNAL( signalDirectoryChanged(const QString &) ), this, SIGNAL( signalValueChanged () ));
  //pixmap = QPixmap( attribute->get_value() );
  this->leftoff = leftoff;
  this->topoff = topoff;
  this->clip = clip;
  this->animated = false;
  updatePixmap();
  connect(attribute, SIGNAL( signalValueChanged(const QString &) ), this, SLOT( updatePixmap() ));
  connect(attribute, SIGNAL( signalDirectoryChanged(const QString &) ), this, SLOT( updatePixmap() ));
}

PreviewItemImage::~PreviewItemImage()
{
}

QString PreviewItemImage::getToolTip() {
  return attribute->get_description();
}


void PreviewItemImage::updatePixmap() {
  if ( ! ( attribute->get_directory().isEmpty() || attribute->get_value().isEmpty() )  ) {
    this->animated = AnimChecker::isAnimatedGIF(attribute->get_directory() + attribute->get_value());
    if ( this->animated && attribute->animation_allowed() ) {
      movie = QMovie( attribute->get_directory() + attribute->get_value() );
      movie.connectUpdate(this, SLOT(movieUpdated(const QRect&)));
    }
    else {
      pixmap = QPixmap( attribute->get_directory() + attribute->get_value() );
      if ( ! attribute->transparency_allowed() ) {
        QBitmap mask = QBitmap(pixmap.size());
        QPainter paint(&mask);
        paint.setBrush(Qt::white);
        paint.drawRect(0,0, mask.width(), mask.height());
        pixmap.setMask(mask);
      }
    }
  }
  else {
    pixmap = 0;
  }
}

void PreviewItemImage::movieUpdated(const QRect&r) {
  emit PreviewItemImage::redraw(r);
}

void PreviewItemImage::draw(QPainter* p)
{
  if ( this->animated ) {
    pixmap = movie.framePixmap();
  }
  if (! pixmap.isNull() /*&& show_img*/ ) {
    if ( clip) {
        int left_displ = std::max( (attribute->get_maxWidth() - pixmap.width() ) / 2, 0);
        int top_displ = std::max( (attribute->get_maxHeight() - pixmap.height() ) / 2, 0);
        p->drawPixmap ( left+left_displ, top+top_displ, pixmap, leftoff, topoff) ;
    }
    else {
        p->drawPixmap ( left, top, pixmap, leftoff, topoff) ;
    }
  }
}

#include "previewitemimage.moc"
