/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "themeattribute.h"

#include "../ui/commandeditattributevalue.h"

#include <klocale.h>


ThemeAttribute::ThemeAttribute(QString xmlName, QString uiName, QString description, bool required, QString value, KCommandHistory *commandHistory ) : QObject() {
  this->xmlName = xmlName;
  this->uiName = uiName;
  this->description = description;
  this->required = required;
  this->type = type;
  this->value = value;
  this->default_value = value;
  this->invalid = false;
  labelWidget = 0;
  editWidget = 0;
  this->commandHistory = commandHistory;
}

ThemeAttribute::ThemeAttribute(const ThemeAttribute& copy) : QObject() {
  this->xmlName = "";
  this->uiName = "";
  this->description = "";
  this->required =false;
  this->type = 0;
  this->value = "";
  this->default_value = "";
  this->invalid = false;
  labelWidget = 0;
  editWidget = 0;
  this->commandHistory = copy.commandHistory;

}



ThemeAttribute::~ThemeAttribute()
{
}


/**
 * Methods
 */
 
void ThemeAttribute::reset( ) {
  value = default_value;
  invalid = false;
  emit signalInvalidChanged(false);
  emit signalValueChanged(value);
  emit signalAttributeChanged(xmlName);
}

/**
 * 
 */
QString ThemeAttribute::get_xmlName ( ) {
  return xmlName;
}
/**
 * 
 */
void ThemeAttribute::set_xmlName (QString value ) {
  xmlName = value;
}
/**
 * 
 */
QString ThemeAttribute::get_uiName ( ) {
  return uiName;
}
/**
 * 
 */
void ThemeAttribute::set_uiName (QString value ) {
  uiName = value;
}
/**
 * 
 */
QString ThemeAttribute::get_description ( ) {
  return description;
}
/**
 * 
 */
void ThemeAttribute::set_description (QString value ) {
  description = value;
}
/**
 * 
 */
bool ThemeAttribute::get_required ( ) {
  return required;
}
/**
 * 
 */
void ThemeAttribute::set_required (bool value ) {
  required = value;
}


QString ThemeAttribute::get_value ( ) {
  return value;
}
/**
 * 
 */
/*void ThemeAttribute::set_value (QString v ) {
  value = v;
  emit signalValueChanged (value);
}*/

void ThemeAttribute::set_value (const QString & v, bool undo) {
  if (value != v) {
    // create command here
    if (undo) {
      commandHistory->addCommand(new CommandEditAttributeValue(this, v, i18n("Change %1").arg(uiName)), false);
    }
/*    invalid = false;
    emit signalInvalidChanged(invalid);*/
    value = v;
    emit signalValueChanged (value);
    emit signalAttributeChanged(xmlName);
  }
}


/**
 * 
 */
void ThemeAttribute::set_invalid (bool value ) {
  //if (invalid != value) {
    invalid = value;
    emit signalInvalidChanged(invalid);
  //}
}

/**
 * 
 */
bool ThemeAttribute::isInvalid () {
  return invalid;  
}


void ThemeAttribute::set_directory (const QString &) {
}





#include "themeattribute.moc"
