/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef THEMEATTRIBUTE_H
#define THEMEATTRIBUTE_H

#include <qobject.h>

class QWidget;
class QLabel;
class KCommandHistory;


/**
@author Matthias Wetzka
*/
class ThemeAttribute : public QObject
{
Q_OBJECT
public:
  enum Type { Color = 1, Image, Text, Version };
  //ThemeAttribute(QString xmlName, int type);
   
  ThemeAttribute(QString xmlName, QString uiName, QString description, bool required, QString value, KCommandHistory *commandHistory );
  ThemeAttribute(const ThemeAttribute&);

  ~ThemeAttribute();
  
  void reset( );

  QString get_xmlName ( );
  void set_xmlName (QString value );
    
  QString get_uiName ( );
  void set_uiName (QString value );
    
  QString get_description ( );
  void set_description (QString value );
    
  bool get_required ( );
  void set_required (bool value );
    
  QString get_value ( );
    
  void set_invalid (bool value );
  bool  isInvalid ();
  
  virtual int get_type() = 0;
  virtual QString createXML() = 0;
  virtual QString getXMLTagAttribute() = 0;
  virtual QWidget *createLabelWidget(QWidget * parent) = 0;
  virtual QWidget *createEditWidget(QWidget * parent) = 0;
  
  virtual void updateSettings() { };
  
  virtual QString get_directory() { return ""; };
public slots:
   virtual void set_directory (const QString &);

public slots:
  virtual void set_value (const QString &, bool undo = true );
  
signals:  
  void  signalValueChanged ( const QString &); // sends changed value
  void  signalAttributeChanged ( const QString &); // sends xmlName of the changed attribute
  void  signalInvalidChanged (bool);
  void signalEnterFocus();
  void signalLostFocus();
  
  
protected:
   QLabel * labelWidget;
   QWidget * editWidget;
   
   QString xmlName;
   QString uiName;
   QString description;
   bool required;
   int type;
   QString value;
   QString default_value;
   bool invalid;
   KCommandHistory *commandHistory;

};

#endif
