/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "themeattributeimage.h"

#include "../ui/commandeditattributedirectory.h"
#include "../ui/mykurlrequester.h"

#include <qsize.h>
#include <qlabel.h>
#include <qwidget.h>
#include <qimage.h>
#include <qtooltip.h>

#include <klistview.h>
#include <kfileitem.h>
#include <kdirwatch.h>
#include <klocale.h>
#include <kurl.h>
#include <kio/netaccess.h>


ThemeAttributeImage::ThemeAttributeImage(QString xmlName, QString uiName, QString description,
  bool required, bool transparent, bool animated, int maxWidth, int maxHeight, QString value,
  KCommandHistory *commandHistory, QString imagefilter, QString directory)
     : ThemeAttribute(xmlName, uiName, description, required, value, commandHistory)
{
  this->transparent = transparent;
  this->animated = animated;
  this->maxWidth = maxWidth;
  this->maxHeight = maxHeight;
  this->directory = directory;
  this->imagefilter = imagefilter;
  this->listViewItem = 0;
  connect(KDirWatch::self(), SIGNAL(dirty (const QString &)), this, SLOT(file_modified( const QString& )));
}


ThemeAttributeImage::~ThemeAttributeImage()
{
}

int ThemeAttributeImage::get_type ( ) {
  return ThemeAttribute::Image;
}


/**
 * 
 */
int ThemeAttributeImage::get_maxWidth ( ) {
  return maxWidth;
}
/**
 * 
 */
void ThemeAttributeImage::set_maxWidth (int value ) {
  maxWidth = value;
}
/**
 * 
 */
int ThemeAttributeImage::get_maxHeight ( ) {
  return maxHeight;
}

bool ThemeAttributeImage::animation_allowed() {
  return animated;
}

bool ThemeAttributeImage::transparency_allowed() {
  return transparent;
}

/**
 * 
 */
void ThemeAttributeImage::set_maxHeight (int value ) {
  maxHeight = value;
}

QSize ThemeAttributeImage::get_maxSize() {
  return QSize(maxWidth, maxHeight);
}

QString ThemeAttributeImage::get_directory() {
  return directory;
}

QString ThemeAttributeImage::get_mimetype() {
  return mimetype;
}

int ThemeAttributeImage::get_size() {
  return size;
}


void ThemeAttributeImage::set_value (const QString & value, bool undo) {
  this->removeDirWatch();
  ThemeAttribute::set_value(value, undo);
  this->addDirWatch();
  updateListViewItem();
}

void ThemeAttributeImage::set_directory (const QString & directory, bool undo) {
  if ( this->directory != directory ) {
    if (undo) {
      commandHistory->addCommand(new CommandEditAttributeDirectory(this, directory, i18n("Change %1").arg(uiName)), false);
    }
    this->removeDirWatch();
    this->directory = directory;
    updateListViewItem();
    emit signalDirectoryChanged(directory);
    emit signalAttributeChanged(xmlName);
    this->addDirWatch();
  }
}

void ThemeAttributeImage::set_fullpath(const QString &value, const QString &dir, bool undo) {
    if (undo) {
      commandHistory->addCommand(new CommandEditAttributeDirectory(this, value, dir, i18n("Change %1").arg(uiName)), false);
    }
    this->removeDirWatch();
    this->directory = dir;
    this->value = value;
    this->addDirWatch();
    updateListViewItem();
    emit signalValueChanged(value);
    emit signalAttributeChanged(xmlName);
    emit signalDirectoryChanged(directory);
}

void ThemeAttributeImage::addDirWatch() {
    if ( ! ( this->value.isEmpty() || this->directory.isEmpty() ) ) {
        KDirWatch::self()->addFile(this->directory+this->value);
    }
}

void ThemeAttributeImage::removeDirWatch() {
    if ( ! ( this->value.isEmpty() || this->directory.isEmpty() ) ) {
        KDirWatch::self()->addFile(this->directory+this->value);
    }
}

void ThemeAttributeImage::file_modified (const QString & filename) {
  if (! value.isEmpty() ) {
    if( filename == directory + value ) {
      emit signalValueChanged(value);
      emit signalAttributeChanged(xmlName);
      emit signalDirectoryChanged(directory);
    }
  }
}


void ThemeAttributeImage::updateListViewItem() {
  if ( listViewItem != 0 ) {
    if ( ! value.isEmpty() ) {
      KFileItem * item = new KFileItem(KFileItem::Unknown, KFileItem::Unknown, KURL(directory + value), true);
      // TODO maybe ist better to use KMimeType::findByFileContent( filename )->name(); ?
      this->mimetype = item->mimeComment();
      this->size = item->size();
      delete item;
      listViewItem->setText(1, value );
      listViewItem->setText(2, this->mimetype);
      listViewItem->setText(3, KIO::convertSize(this->size));
      QString dim = "";
      QImage *img = new QImage( directory + value );
      if ( ! img->isNull () ) {
        dim.sprintf("%d x %d ", img->width(), img->height());
      }
      delete img;
      listViewItem->setText(4, dim);
      listViewItem->setVisible(true);
    }
    else {
      listViewItem->setVisible(false);
    }
  }
}


QString ThemeAttributeImage::createXML() {
  if ( value != QString::null ) {
    return "<"+xmlName+" Source=\""+value+"\"/>";
  }
  else {
    return QString::null;
  }
}

QWidget *ThemeAttributeImage::createLabelWidget(QWidget * parent) {
 if (labelWidget == 0) {
   labelWidget = new QLabel(QString("%1 (%2 x %3)").arg(uiName).arg( maxWidth ).arg( maxHeight ), parent, "l"+xmlName);
   labelWidget->setAlignment(Qt::AlignLeft | Qt::AlignVCenter | Qt::ExpandTabs);
   //labelWidget->setBackgroundColor( Qt::red );
 }
 return labelWidget;
}

QWidget *ThemeAttributeImage::createEditWidget(QWidget * parent) {
  if (editWidget == 0) {
    editWidget = new MyKURLRequester(parent, "urlRequester"+xmlName, imagefilter, this);
  }
  return editWidget;
}

QListViewItem * ThemeAttributeImage::create_listViewItem(KListView * listView) {
  if (listViewItem == 0) {
    listViewItem = new QListViewItem(listView);
    listViewItem->setText(0, xmlName);
    listViewItem->setVisible(false);
    updateListViewItem();
  }
  return listViewItem;
}

void ThemeAttributeImage::updateSettings() {
  ((MyKURLRequester*)editWidget)->updateTooltip();
}

bool ThemeAttributeImage::copyTo(const KURL &url, QWidget * window) {
  KURL src;
  src.setPath(this->directory + this->value);
  if ( src.isValid() ) {
    return KIO::NetAccess::file_copy( src, url, -1, true, false, window);
    //return KIO::NetAccess::copy( src, url, window);
  }
  else {
    return false;
  }
}

#include "themeattributeimage.moc"
