/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef THEMEATTRIBUTEIMAGE_H
#define THEMEATTRIBUTEIMAGE_H

#include "themeattribute.h"
#include <qobject.h>

class QSize;
class KCommandHistory;
class QListViewItem;
class KListView;
class KURL;

/**
@author Matthias Wetzka
*/
class Q_EXPORT ThemeAttributeImage : public ThemeAttribute
{
Q_OBJECT
public:
    ThemeAttributeImage(QString xmlName, QString uiName, QString description,
      bool required, bool transparent, bool animated, int maxWidth, int maxHeight,
      QString value, KCommandHistory *commandHistory, QString imagefilter, QString directory = QString::null);

    ~ThemeAttributeImage();
    
    virtual int get_type ( );
    virtual QString createXML();
    virtual QString getXMLTagAttribute() { return "Source"; }
    virtual QWidget *createLabelWidget(QWidget * parent);
    virtual QWidget *createEditWidget(QWidget * parent);
    virtual void updateSettings();

    int get_maxWidth ( );
    void set_maxWidth (int value );
    int get_maxHeight ( );
    void set_maxHeight (int value );
    QSize get_maxSize();
    QString get_mimetype();
    int get_size();
    bool animation_allowed();
    bool transparency_allowed();
    
    QString get_directory();
    QListViewItem * create_listViewItem(KListView * listView);
    void set_fullpath(const QString &, const QString &, bool undo = true);
    bool copyTo(const KURL&, QWidget * window = 0L);
    
signals:
    void signalDirectoryChanged(const QString & );
    
public slots:
    virtual void set_directory (const QString &, bool undo = true);
    virtual void set_value (const QString &, bool undo = true );
    void file_modified (const QString &);

    
protected:
   int maxWidth;
   int maxHeight;
   bool transparent;
   bool animated;
   QString directory;
   QString imagefilter;
   QListViewItem * listViewItem;
   QString mimetype;
   int size;
   void updateListViewItem();
   void addDirWatch();
   void removeDirWatch();

};

#endif
