//
// C++ Implementation: themechecker
//
// Description: 
//
//
// Author: Matthias Wetzka <matthias.wetzka@gmx.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "themechecker.h"

#include "themeattribute.h"
#include "themefilemodel.h"

#include <qcolor.h>
#include <qptrlist.h>
#include <qstringlist.h>


ThemeChecker::ThemeChecker(ThemeFileModel *model)
 : QObject()
{
  this->model = model;
}


ThemeChecker::~ThemeChecker()
{
}


void ThemeChecker::setColorDependencies(const QMap<QString, QStringList> &colorDependencyMap) {
  this->colorDependencyMap = colorDependencyMap;
  QMap<QString, QStringList>::Iterator it;
  for ( it = this->colorDependencyMap.begin(); it != this->colorDependencyMap.end(); ++it ) {
    ThemeAttribute *attribute = model->getAttribute(it.key());
    violatedColorDependencies[attribute] = QPtrList<ThemeAttribute>();
  }
}


void ThemeChecker::slot_valueChanged(const QString & attr_name) {
  ThemeAttribute *attribute = model->getAttribute(attr_name);
  // this is connected to every attribute
  checkColorDependencies(attribute);
}


void ThemeChecker::checkColorDependencies(ThemeAttribute *attribute_1) {
  //attribute_1->set_invalid(false);
  violatedColorDependencies[attribute_1].clear();
  //QPtrList<ThemeAttribute> invalids;
  //iterate through color dependent attributes
  QStringList deps = colorDependencyMap[attribute_1->get_xmlName()];
  for ( QStringList::Iterator it = deps.begin(); it != deps.end(); ++it ) {
    ThemeAttribute *attribute_2 = model->getAttribute(*it);
    violatedColorDependencies[attribute_1].clear();
    //attribute_2->set_invalid(false);
    if ( calculateRGBDistance( QColor("#"+attribute_1->get_value()), QColor("#"+attribute_2->get_value()) ) <= 12 ) {
      if ( violatedColorDependencies[attribute_1].find(attribute_2) == -1 ) {
        violatedColorDependencies[attribute_1].append(attribute_2);
      }
      if ( violatedColorDependencies[attribute_2].find(attribute_1) == -1 ) {
        violatedColorDependencies[attribute_2].append(attribute_1);
      }
      emit signal_attributeViolation(VIOL_COLOR_DISTANCE, attribute_1, attribute_2);
      //invalids.append(attribute_2);
    } else {
      //if ( violatedColorDependencies[attribute_1].find(attribute_2) != -1 ) {
        violatedColorDependencies[attribute_1].remove(attribute_2);
      //}
//       if ( violatedColorDependencies[attribute_2].find(attribute_1) != -1 ) {
        violatedColorDependencies[attribute_2].remove(attribute_1);
//       }
    }
  }
  // iterate through invalid and emit signals
  
  QMap<ThemeAttribute *, QPtrList<ThemeAttribute> >::Iterator it;
  bool noneInvalid = true;
  for ( it = this->violatedColorDependencies.begin(); it != this->violatedColorDependencies.end(); ++it ) {
    ThemeAttribute *attribute = it.key();
    if ( violatedColorDependencies[attribute].isEmpty() ) {
      //cout << attribute->get_xmlName() << " is empty, setting it to false." << endl;
      attribute->set_invalid( false );
    }
    else {
      //cout << attribute->get_xmlName() << " is NOT empty, setting it to true!" << endl;
      attribute->set_invalid( true );
      noneInvalid = false;
    }
  }
  this->model->setInvalid( ! noneInvalid );
/*  ThemeAttribute *invalid;
  for ( invalid = invalids.first(); invalid; invalid = invalids.next() ) {
    if ( violatedColorDependencies[attribute_1].find(invalid) == -1 ) {
      violatedColorDependencies[attribute_1].append(invalid);
    }
    if ( violatedColorDependencies[invalid].find(attribute_1) == -1 ) {
      violatedColorDependencies[invalid].append(attribute_1);
    }

    //emit signal_attributeViolation(VIOL_COLOR_DISTANCE, attribute_1, invalid);
  }
  // connect with slot of statusbar or messagelist
  
  //getAttribute(attribute)->set_invalid(isInvalid);*/
}

/*
bool ThemeChecker::checkColorDependency(ThemeAttribute * att_1, ThemeAttribute * att_2) {
  if (  ) {
    //a2->set_invalid(true);
    return true;
  }
  else {
    //a2->set_invalid(false);
    return false;
  }
}*/


double ThemeChecker::calculateRGBDistance(const QColor &c1, const QColor &c2) {
  int rd = c1.red() - c2.red();
  int gd = c1.green() - c2.green();
  int bd = c1.blue() - c2.blue();
  return sqrt(rd*rd + gd*gd + bd*bd);
}


#include "themechecker.moc"