/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "themedefinitionloader.h"

#include "themekreator.h"
#include "themeviewwidget.h"
#include "themefilemodel.h"
#include "themechecker.h"
#include "previewdatamodel.h"
#include "previewitemcolor.h"
#include "previewitemimage.h"
#include "previewitempixmap.h"
#include "uimodel.h"
#include "themeattributecolor.h"
#include "themeattributeimage.h"
#include "themeattributetext.h"
#include "themeattributeversion.h"
#include "mylineedit.h"

#include <klocale.h>
#include <kcommand.h>
#include <kstddirs.h>
#include <kmessagebox.h>

#include <qfile.h>
#include <qdatetime.h>
#include <qstring.h>

ThemeDefinitionLoader::ThemeDefinitionLoader(ThemeKreator * themekreator, ThemeViewWidget * themeview)
{
  m_themekreator = themekreator;
  m_themeview = themeview;
  theme_checker = m_themekreator->getThemeChecker();
  m_screenwidth = 0;
  m_screenheight = 0;
  uiModel = 0;
  previews.setAutoDelete(true);
}

void ThemeDefinitionLoader::load(const QString &phonemodel) {
 /* if model equals oldmodel, just reset() all Components 
   this only makes sense if one window can _ever_ be used to edit two different Themes -> SDI <-> MDI
 */
//  KCommandHistory * commandHistory = m_themekreator->commandHistory;
  QString filename = locate("appdata", phonemodel+"/"+phonemodel+".thd");
  QDomDocument m_doc( "theme definition" );
  //KMessageBox::sorry(m_themekreator, i18n("Opening File %1. ").arg(filename));
  QFile file( filename );
  if ( !file.open( IO_ReadOnly ) ) {
    KMessageBox::sorry(m_themekreator, i18n("Could not read theme definition for %1. Aborting.").arg(phonemodel));
    exit(0);
    return;
  }
  QString * errorMsg = new QString();// = new QString();
  int * errorLine = new int();
  int * errorCol = new int();
  if ( !m_doc.setContent( &file, errorMsg, errorLine , errorCol ) ) {
      file.close();
      KMessageBox::sorry(m_themekreator, i18n("Could not load theme definition for %1. Error %2 in line %3, column %4. Aborting.").arg(phonemodel).arg(*errorMsg).arg(*errorLine).arg(*errorCol));
      delete errorMsg;
      delete errorLine;
      delete errorCol;
      exit(0);
  }
  delete errorMsg;
  delete errorLine;
  delete errorCol;
  file.close();

  uiModel = new UIModel();
  QDomElement root = m_doc.documentElement();
  QDomNode node;
  
  /* SETTINGS */
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "settings" ) {
      QDomElement settings = node.toElement();
      getSettingsInformation( settings );
      break;
    }
    node = node.nextSibling();
  }
  
  
  /* ATTRIBUTES */  
  attributes.resize(199); // max 199 attributes processable in hash
  attributes.setAutoDelete(true);
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "attributes" ) {
      QDomElement attributes = node.toElement();
      getAttributesInformation( attributes );
      break;
    }
    node = node.nextSibling();
  }
  
  appendETTAttributes();

  /* DEPENDENCIES */  
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "colordependencies" ) {
      QDomElement dependencies = node.toElement();
      getDependenciesInformation( dependencies );
      break;
    }
    node = node.nextSibling();
  }

  /* PREVIEWS */
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "previews" ) {
      QDomElement previews = node.toElement();
      getPreviewsInformation( previews, phonemodel );
      break;
    }
    node = node.nextSibling();
  }
  
  /* USER INTERFACE */
  node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "userinterface" ) {
      QDomElement userinterface = node.toElement();
      getUIInformation( userinterface );
      break;
    }
    node = node.nextSibling();
  }

  m_themekreator->getFileModel()->setThemeVersion(m_version);
  m_themekreator->getFileModel()->setImageFormats(m_imageformats);
  m_themekreator->getFileModel()->setAttributes(attributes);
  theme_checker->setColorDependencies(dependencyMap);
  //m_themekreator->getFileModel()->setDependencies(dependencyMap);
  m_themekreator->getFileModel()->getAttribute("Phone_model")->set_value( phonemodel );
  //m_themekreator->m_view->setFileModel(m_themekreator->getFileModel());
  
  //fileModel->setVersion(version);
  
  //m_themekreator->setFileModel( fileModel );
  m_themekreator->getFileModel()->setPreviews(&previews);
  m_themekreator->getFileModel()->setPreviewNames(preview_names);
  m_themeview->setUIModel( uiModel );
}

QString ThemeDefinitionLoader::getValue( const QDomElement & root, const QString &nodeName ) {
  /*QDomElement tab = node.toElement();
  QString tabname;*/
  QDomNode child = root.firstChild();      
  while ( !child.isNull() ) {
    if ( child.isElement() && child.nodeName() == nodeName ) {
       QDomText textChild = child.firstChild().toText();
       if ( !textChild.isNull() ) {
         return textChild.nodeValue();
       }
       else {
          // parse error: title mus not be null
       }
       break;
    }
    child = child.nextSibling();
  }
  return QString::null;
}

QString ThemeDefinitionLoader::getTextValue( const QDomElement & root, const QString &nodeName ) {
  QDomNode child = root.firstChild();      
  QString enText;
  QString localText;
  while ( !child.isNull() ) {
    if ( child.isElement() && child.nodeName() == nodeName ) {
      QDomNode element = child.toElement().firstChild();
      return i18n(element.toText().nodeValue());
    
/*      QDomNode translation = child.toElement().firstChild();
      while ( !translation.isNull() ) {
              if ( translation.isElement() && translation.nodeName() == "i18n" ) {
          if ( translation.toElement().attribute("lang") == "en" ) {
            enText = translation.firstChild().toText().nodeValue();
          }
          if ( translation.toElement().attribute("lang") == "de" ) {
            localText = translation.firstChild().toText().nodeValue();
          }
          //QDomText localText = ;
          //if ( !textChild.isNull() ) {
          //return localText.nodeValue();
          if ( localText != QString::null ) {
            break;
          }        
        }
        translation = translation.nextSibling();  
      }
      if ( localText != QString::null ) { 
        return localText;
      }
      else {
        return enText;
      }*/
    }
    child = child.nextSibling();
  }
  return QString::null;
}

void ThemeDefinitionLoader::getSettingsInformation( const QDomElement & root ) {
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() ) {
      if (node.nodeName() == "version" ) {
        m_version = node.toElement().firstChild().toText().nodeValue();
      }
      else if (node.nodeName() == "model" ) {
        //m_phonemodel = node.toElement().firstChild().toText().nodeValue();
      }
      else if (node.nodeName() == "screen" ) {
        m_screenheight = node.toElement().attribute( "height" ).toInt();
        m_screenwidth = node.toElement().attribute( "width" ).toInt();
      }
      else if (node.nodeName() == "imgformats" ) {
        m_imageformats = node.toElement().firstChild().toText().nodeValue();
      }
      else if (node.nodeName() == "colors" ) {
        /* not implemented, yet */
      }
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getAttributesInformation( const QDomElement & root ) {
  // we pares this dom node two time
  // first, we collect all images, second we collect all other attributes
  // using the qmap, we can add the depends_on attribute to the colors
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "attribute" ) {
      QString required  = node.toElement().attribute( "required" );
      QString type  = node.toElement().attribute( "type" );
      QString xmlName = getValue( node.toElement(), "xmlname" );
      // TODO: check these integer
      int maxwidth = node.toElement().attribute( "maxwidth" ).toInt();;
      int maxheight = node.toElement().attribute( "maxheight" ).toInt();;
      QString anim_allowed  = node.toElement().attribute( "anim" );
      QString trans_allowed  = node.toElement().attribute( "trans" );
      
      ThemeAttribute *attribute = 0;
      if ( type == "image" ) {
        attribute = new ThemeAttributeImage(xmlName,
          getTextValue( node.toElement(), "uiname" ), 
          getTextValue( node.toElement(), "description" ), 
          required == "true",
          trans_allowed == "true",
          anim_allowed == "true",
          maxwidth,
          maxheight, 
          getValue( node.toElement(), "default" ), 
          m_themekreator->commandHistory,
          m_imageformats);
      }
      else if (type == "color") {
        attribute = new ThemeAttributeColor(
          xmlName,
          getTextValue( node.toElement(), "uiname" ),
          getTextValue( node.toElement(), "description" ),
          required == "true",
          getValue( node.toElement(), "default" ),
          m_themekreator->commandHistory);
      }
      else if (type == "text") {
        attribute =  new ThemeAttributeText(
          xmlName,
          getTextValue( node.toElement(), "uiname" ),
          getTextValue( node.toElement(), "description" ),
          required == "true",
          getValue( node.toElement(), "default" ),
          m_themekreator->commandHistory);
      }
      else if (type == "version") {
        /*attributes.insert( xmlName, new ThemeAttributeVersion(
          xmlName,
          getTextValue( node.toElement(), "uiname" ),
          getTextValue( node.toElement(), "description" ),
          required == "true",
          getValue( node.toElement(), "default" ),
          m_themekreator->commandHistory)
        );*/
      }
      if ( attribute != 0) {
        attributes.insert(xmlName, attribute);
        connect(attribute, SIGNAL(signalAttributeChanged( const QString &)), theme_checker, SLOT(slot_valueChanged( const QString & )));
      }
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getUIInformation( const QDomElement & root ) {
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "tab" ) {
      QDomElement tab = node.toElement();
      PreviewDataModel * preview = 0;
      if ( ! node.toElement().attribute( "preview" ).isNull() ) {
        QString name = node.toElement().attribute( "preview" );
        preview = previews[name];
        preview_names.append(name);
      }
      UITab * uiTab = new UITab( getTextValue( tab, "title" ), preview );
      getUITabInformation( tab, uiTab );
      uiModel->append( uiTab );
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getUITabInformation( const QDomElement & root, UITab * uiTab ) {
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "group" ) {
      int cols = node.toElement().attribute("cols").toInt();
      if ( cols == 0 ) cols = 2;
      QDomElement group = node.toElement();
      UIGroup * uiGroup = new UIGroup( getTextValue( group, "title" ), cols * 2 );
      getUIGroupInformation( group, uiGroup );
      uiTab->append( uiGroup );
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getUIGroupInformation( const QDomElement & root, UIGroup * uiGroup ) {
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "item" ) {
      uiGroup->append( attributes[node.toElement().attribute("attribute")] );
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getDependenciesInformation( const QDomElement & root ) {
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "dependency" ) {
      QString attribute = node.toElement().attribute("attribute");
      dependencyMap[ attribute ] = QStringList() ;
      
      QDomNode child = node.firstChild();
      while ( !child.isNull() ) {
        if ( child.isElement() && child.nodeName() == "dependant" ) {
          dependencyMap[ attribute ].append( child.toElement().attribute("attribute") );
        }
        child = child.nextSibling();
      }
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getPreviewsInformation( const QDomElement & root, QString phonemodel ) {
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() && node.nodeName() == "preview" ) {
      QString name = node.toElement().attribute("name");
      PreviewDataModel * preview = new PreviewDataModel(m_screenwidth, m_screenheight);
      previews.insert( name, preview ); // TODO: replace width and height
      getPreviewItemsInformation( node.toElement(), preview, phonemodel );
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getPreviewItemsInformation( const QDomElement & root, PreviewDataModel * preview, QString phonemodel ) {
  QDomNode node = root.firstChild();
  while ( !node.isNull() ) {
    if ( node.isElement() ) {
      if ( node.nodeName() == "color") getPreviewColorInformation( node.toElement(), preview, phonemodel );
      else if ( node.nodeName() == "image") getPreviewImageInformation( node.toElement(), preview );
      else if ( node.nodeName() == "pixmap") getPreviewPixmapInformation( node.toElement(), preview, phonemodel );
    }
    node = node.nextSibling();
  }
}

void ThemeDefinitionLoader::getPreviewColorInformation( const QDomElement & root, PreviewDataModel * preview, QString phonemodel ) {
  // TODO: alpha channel
  ThemeAttribute * attribute = 0;
  attribute = attributes[ root.attribute( "attribute" ) ];
  int alpha = root.attribute( "alpha", "100" ).toInt(); // TODO check parse if int
  int left = root.attribute( "left", "0" ).toInt(); // TODO check parse if int
  int top = root.attribute( "top", "0" ).toInt(); // TODO check parse if int
  int width = root.attribute( "width", "0" ).toInt(); // TODO check parse if int
  int height = root.attribute( "height", "0" ).toInt(); // TODO check parse if int
  QString depends_on = root.attribute( "depends_on", QString::null );
  ThemeAttribute * attr_depends_on = 0;
  if ( ! depends_on.isNull() ) {
    attr_depends_on = attributes[depends_on];
  }
  QString bitmap = phonemodel+"/" + root.attribute( "bitmap" );
  if (attribute != 0) {
    preview->append( new PreviewItemColor(left, top, width, height, attribute, alpha, bitmap, attr_depends_on) );
  }
  else {
  }
}

void ThemeDefinitionLoader::getPreviewImageInformation( const QDomElement & root, PreviewDataModel * preview ) {
  ThemeAttribute * attribute = 0;
  attribute = attributes[ root.attribute( "attribute" ) ];
  int left = root.attribute( "left", "0" ).toInt(); // TODO check parse if int
  int top = root.attribute( "top", "0" ).toInt(); // TODO check parse if int
  int width = root.attribute( "width", "-1" ).toInt(); // TODO check parse if int
  int height = root.attribute( "height", "-1" ).toInt(); // TODO check parse if int
  int leftoff = root.attribute( "leftoff", "0" ).toInt(); // TODO check parse if int
  int topoff = root.attribute( "topoff", "0" ).toInt(); // TODO check parse if int
  bool clip = ("true" == root.attribute( "clip", "true" ) );
  if (attribute != 0) {
    preview->append( new PreviewItemImage(left, top, width, height, attribute, leftoff, topoff, clip ) );
  }
  else {
  }

}

void ThemeDefinitionLoader::getPreviewPixmapInformation( const QDomElement & root, PreviewDataModel * preview, QString phonemodel ) {
  QString src = phonemodel+"/" + root.attribute( "src" );
  //KMessageBox::sorry(m_themekreator, i18n("Opening File %1. ").arg(src));
  int left = root.attribute( "left", "0" ).toInt(); // TODO check parse if int
  int top = root.attribute( "top", "0" ).toInt(); // TODO check parse if int
  QString depends_on = root.attribute( "depends_on", QString::null );
  ThemeAttribute * attr_depends_on = 0;
  if ( ! depends_on.isNull() ) {
    attr_depends_on = attributes[depends_on];
  }
  preview->append( new PreviewItemPixmap(left, top, src, attr_depends_on) );
}


//   <Author_firstname value="Matthias"/>
//   <Author_lastname value="Wetzka"/>
//   <Author_organisation value="MettySoft"/>
//   <Author_email value="m@w.de"/>
//   <Author_url value="www.matthiaswetzka.de"/>


void ThemeDefinitionLoader::appendETTAttributes() {
  attributes.insert( "Phone_model", new ThemeAttributeText(
          "Phone_model", 
          i18n("Phone model"), 
          i18n("The model of the phone this theme was written for."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );
/*  attributes.insert( "Theme_name", new ThemeAttributeText(
          "Theme_name", 
          i18n("Theme name"), 
          i18n("The name of the theme."), 
          true, 
          i18n("unnamed"), 
          m_themekreator->commandHistory)
  );
  attributes.insert( "Theme_creator", new ThemeAttributeText(
          "Theme_creator", 
          i18n("Created by"), 
          i18n("Name and/or e-mail addres of person who created this theme."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );
  attributes.insert( "Theme_modified_by", new ThemeAttributeText(
          "Theme_modified_by", 
          i18n("Modified by"), 
          i18n("Name and/or e-mail addres of person who modified this theme."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );
  attributes.insert( "Theme_created", new ThemeAttributeText(
          "Theme_created", 
          i18n("Creation date"), 
          i18n("Date and time when the theme was created."), 
          true, 
          QDateTime::currentDateTime().toString("d.M.yyyy hh:mm:ss"),
          m_themekreator->commandHistory)
  );*/
  attributes.insert( "Author_firstname", new ThemeAttributeText(
          "Author_firstname",
          i18n("First Name"),
          i18n("First name of the author of this theme."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );
  attributes.insert( "Author_lastname", new ThemeAttributeText(
          "Author_lastname",
          i18n("Last Name"),
          i18n("Last name of the author of this theme."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );
  attributes.insert( "Author_organisation", new ThemeAttributeText(
          "Author_organisation",
          i18n("Organization"),
          i18n("Organization of the author of this theme."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );
  attributes.insert( "Author_email", new ThemeAttributeText(
          "Author_email",
          i18n("Email"),
          i18n("Email address of the author of this theme."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );
  attributes.insert( "Author_url", new ThemeAttributeText(
          "Author_url",
          i18n("Website"),
          i18n("Website of the author of this theme."), 
          true, 
          "", 
          m_themekreator->commandHistory)
  );

}


ThemeDefinitionLoader::~ThemeDefinitionLoader()
{
  if (uiModel != 0) { delete uiModel; }
  previews.clear();
}


#include "themedefinitionloader.moc"
