/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef THEMEDEFINITIONLOADER_H
#define THEMEDEFINITIONLOADER_H

#include <qobject.h>
#include <qdict.h>
#include <qmap.h>
#include <qstringlist.h>

class QString;

class KCommandHistory;

class ThemeKreator;
class ThemeViewWidget;
class ThemeFileModel;
class ThemeChecker;
class PreviewDataModel;
class UIModel;
class UITab;
class UIGroup;
class ThemeAttribute;
class QDomElement;



/**
@author Matthias Wetzka
*/
class ThemeDefinitionLoader : public QObject{
Q_OBJECT
public:
    ThemeDefinitionLoader(ThemeKreator * themekreator, ThemeViewWidget * themeview);
    
    void load(const QString &phonemodel);
    
    /*ThemeFileModel * getFilemodel();
    PreviewDataModel * getPreviewmodel();
    UIModel * getUImodel();*/

    ~ThemeDefinitionLoader();
private:
  ThemeKreator * m_themekreator;
  ThemeChecker * theme_checker;
  ThemeViewWidget * m_themeview;
  
  QDict<ThemeAttribute> attributes;
  QDict<PreviewDataModel> previews;
  QStringList preview_names;
  QMap<QString, QStringList> dependencyMap;
  
  UIModel * uiModel;
  
  int m_screenwidth;
  int m_screenheight;
  QString m_version;
  QString m_imageformats;
  //PreviewDataModel * previewModel;
  
  //QString getTitleInformation( const QDomElement & root );
  QString getValue( const QDomElement & root, const QString &nodeName );
  QString getTextValue( const QDomElement & root, const QString &nodeName );
  
  void getUIInformation( const QDomElement & root );
  void getUITabInformation( const QDomElement & root, UITab * uiTab );
  void getUIGroupInformation( const QDomElement & root, UIGroup * uiGroup );
  void getSettingsInformation( const QDomElement & root );
  void getAttributesInformation( const QDomElement & root );
  void getDependenciesInformation( const QDomElement & root );
  void getPreviewsInformation( const QDomElement & root, QString phonemodel );
  void getPreviewItemsInformation( const QDomElement & root, PreviewDataModel * preview, QString phonemodel );
  void getPreviewColorInformation( const QDomElement & root, PreviewDataModel * preview, QString phonemodel );
  void getPreviewImageInformation( const QDomElement & root, PreviewDataModel * preview );
  void getPreviewPixmapInformation( const QDomElement & root, PreviewDataModel * preview, QString phonemodel );

  void appendETTAttributes();
};

#endif
