/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "themefilemodel.h"

#include "selectmodeldialog.h"
#include "phonemodelloader.h"
#include "../config/themekreatorconfig.h"
#include "themeattributecolor.h"
#include "themeattributeimage.h"


#include <qwidget.h>
#include <qsize.h>
#include <qcolor.h>
#include <qfile.h>
#include <qobject.h>
#include <qstring.h>
#include <qdir.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qregexp.h>

#include <klocale.h>
#include <kcommand.h>
#include <klistview.h>
#include <kinputdialog.h>
#include <klocale.h>
#include <ktempdir.h> 
#include <ktar.h>
#include <ktempfile.h>
#include <kstddirs.h>
#include <kmessagebox.h>
#include <kconfig.h>


ThemeFileModel::ThemeFileModel()
 : QObject()
{
  m_url = KURL(i18n("unknown.thm"));
  temp_dir = 0;
  lvi_xmlfilename = 0;
  temp_xmlfile = QString::null;
  

  modified = false;
  setModified(false); // just to be sure and to cause events
  invalid = false;
  setInvalid(false);
  
  reset();
}

ThemeFileModel::~ThemeFileModel()
{
  delete temp_dir;
  temp_dir = 0;
}

void ThemeFileModel::setURL(const KURL & url) {
  this->m_url = url;
}

KURL ThemeFileModel::getURL() {
  return this->m_url;
}

void ThemeFileModel::setThemeVersion(QString &version) {
  this->m_version = version;
  this->setModified(true);
}

void ThemeFileModel::setImageFormats(QString &formats) {
  this->m_imageformats = formats;
  this->setModified(true);
}

void ThemeFileModel::setAttributes(const QDict<ThemeAttribute> &attributes) {
  this->attributes = attributes;
  QDictIterator<ThemeAttribute> it( attributes );
  for( ; it.current(); ++it ) {
    //connect (it.current(), SIGNAL(signalAttributeChanged(const QString &)), this, SLOT(checkDependencies(const QString &)));
    connect (this, SIGNAL(dirChanged(const QString &)), it.current(), SLOT(set_directory (const QString &)));
    connect ( it.current(), SIGNAL(signalValueChanged(const QString &)), this, SLOT( handleModified() ) );
  }

  if ( this->attributes["Author_firstname"] != 0) {
    this->attributes["Author_firstname"]->set_value( ThemeKreatorConfig::firstName() );
  }
  if ( this->attributes["Author_lastname"] != 0) {
    this->attributes["Author_lastname"]->set_value( ThemeKreatorConfig::lastName() );
  }
  if ( this->attributes["Author_organisation"] != 0) {
    this->attributes["Author_organisation"]->set_value( ThemeKreatorConfig::organization());
  }
  if ( this->attributes["Author_email"] != 0) {
    this->attributes["Author_email"]->set_value( ThemeKreatorConfig::email() );
  }
  if ( this->attributes["Author_url"] != 0) {
    this->attributes["Author_url"]->set_value( ThemeKreatorConfig::website() );
  }
}


void ThemeFileModel::handleModified() {
  this->setModified(true);
}

void ThemeFileModel::unsetModified() {
  this->setModified(false);
}


QString ThemeFileModel::getAttributeValue(QString attribute) {
  return attributes[attribute]->get_value();
}

/*void ThemeFileModel::setAttributeValue(QString attribute, QString value) {
  attributes[attribute]->set_value(value);
  checkDependencies(attribute);
  this->setModified(true);
}*/

ThemeAttribute *ThemeFileModel::getAttribute(QString attribute) {
  return attributes[attribute];
}

QDict<ThemeAttribute> *ThemeFileModel::getAttributes() {
  return &attributes;
}


bool ThemeFileModel::isModified() {
  return this->modified;
}

void ThemeFileModel::setModified(bool modified) {
  if ( this->modified != modified ) {
    this->modified = modified;
    emit modifiedChanged(modified);
  }
}

bool ThemeFileModel::isInvalid() {
  return this->invalid;
}


void ThemeFileModel::setInvalid(bool invalid) {
  if ( this->invalid != invalid ) {
    this->invalid = invalid;
    emit invalidChanged( invalid );
  }
}


void ThemeFileModel::reset() {
  QDictIterator<ThemeAttribute> it( attributes );
  for( ; it.current(); ++it ) {
    it.current()->reset();
  }
  
//  file_list.clear();
  dir = "";
  m_url = KURL(i18n("unknown.thm"));
  
  emit valueChanged();
  emit dirChanged(dir);
  this->setModified(false);
}

QString ThemeFileModel::asXML() {
  QString xml = "<?xml version=\"1.0\"?>\r\n";
  if ( this->m_version == "1.0" ) {
    xml += "<Ericsson_theme>";
  }
  else {
    xml += "<Sony_Ericsson_theme version=\""+m_version+"\">";
  }
  if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
/*  if ( ThemeKreatorConfig::support_ett() ) {
    xml += "<!--\n";
    xml += asTag("Phone_model", "Name", this->getAttribute("Phone_model")->get_value() );
    if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
    xml += asTag("Theme_name", "Name", this->getAttribute("Theme_name")->get_value() );
    if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
    xml += asTag("Theme_creator", "Name", this->getAttribute("Theme_creator")->get_value() );
    if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
    xml += asTag("Theme_created", "Name", this->getAttribute("Theme_created")->get_value() );
    if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
    xml += asTag("Theme_modified_by", "Name", this->getAttribute("Theme_modified_by")->get_value() );
    if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
    xml += "-->";
    if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
  }*/
  
  QDictIterator<ThemeAttribute> it( attributes );
  for( ; it.current(); ++it ) {
    //if ( it.current()->get_type() != ThemeAttribute::Text ) {
      xml += it.current()->createXML();
      if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
      //asTag(it.current()->get_xmlName(), "Source", QFileInfo(it.current()->get_value()).fileName());
      //xml += asTag(it.current()->get_xmlName(), "Color", "0x"+it.current()->get_value());
    //}
  }

  if ( this->m_version == "1.0" ) {
    xml += "</Ericsson_theme>";
  }
  else {
    xml += "</Sony_Ericsson_theme>";
  }
  if ( ! ThemeKreatorConfig::smallfile() ) { xml += "\r\n"; }
  return xml;
}

void ThemeFileModel::createListViewItems(KListView * listView) {
/*  if ( lvi_xmlfilename == 0 ) {
    lvi_xmlfilename = new QListViewItem(listView, m_filename, QString("XML"), QString(""), QString::null, i18n("Theme description"));
    connect ( this->getAttribute("Theme_name"), SIGNAL(signalAttributeChanged(const QString &)), this, SLOT(slotThemeNameChanged()));
    slotThemeNameChanged();
  }*/
  QDictIterator<ThemeAttribute> it( attributes );
  ThemeAttributeImage *image ;
  for( ; it.current(); ++it ) {
    if (it.current()->get_type() == ThemeAttribute::Image) {
      image = (ThemeAttributeImage *) it.current();
      QListViewItem *item = image->create_listViewItem(listView);
      file_list[ image->get_xmlName() ] = item;
        //connect(it.current(), SIGNAL(signalAttributeChanged(const QString &)), this, SLOT(updateImageListItem(const QString &)));
    }
  }
}


QString ThemeFileModel::asTag(QString tag, QString attribute, QString value) {
  if (value.length() > 0) {
    return "<"+tag+" "+attribute+"=\""+value+"\"/>\r\n";
  }
  else {
    return "";
  }
}

QString ThemeFileModel::getETTModel(QWidget *parent) {
  QString phone_model = getDOMValue("Phone_model", "value");
  if ( phone_model.length() == 0 ) {
    phone_model = getDOMValue("Phone_model", "Name"); // original ETT style
  }
  if (phone_model.length() == 0) { return "unknown"; }

  
  PhoneModelLoader * loader = new PhoneModelLoader();
  QString tk_model = loader->findModel(phone_model);
  delete loader;
  return tk_model;

//   PhoneModelGroup * group = loader->getGroup(version);

/*  KConfig * phoneModelConfig = new KConfig ("phonemodels.lst", true, true, "appdata");
  QStringList models = phoneModelConfig->readListEntry( "models" );
  delete phoneModelConfig;
  QStringList implemented_models = models.grep(ett_model, false);
  if (implemented_models.count() > 1) {
    QString model = KInputDialog::getItem(
      i18n("Select Phone Model"),
      i18n("The exact phone model, for which this theme was designed, could not be determined.\nPlease select a device from the list below."),
      implemented_models
    );
    return model;
  }
  else if (implemented_models.count() == 1) {
    return implemented_models[0];
  }*/
  //return "unknown";
}

QString ThemeFileModel::getModelFromVersion(QWidget *parent) {
  /*
  Version1: tag Ericsson_theme, no version attribute
  Version2: tag Sony_Ericsson_theme, no version attribute
  Version3: tag Sony_Ericsson_theme, version attribute present
  */
  QString version = QString::null;
  QDomNodeList nl;
  nl = m_domdoc.elementsByTagName("Ericsson_theme");
  if (nl.count() > 0) {
    // Version 1
    version = "1.0";
  }
  else {
    nl = m_domdoc.elementsByTagName("Sony_Ericsson_theme");
    if (nl.count() > 0) {
      // Version > 1.0
      QDomNamedNodeMap nnm = nl.item(0).attributes();
      if (nnm.namedItem("version").toAttr().value() != QString::null) {
        // read version 
        version = nnm.namedItem("version").toAttr().value();
      }
      else {
        // version 2 since no version attribute found
        //version = "1.0_2.0";
        version = "2.0";
      }
    }
  }
  
  if (version == QString::null) return QString::null;

  return showModelSelecionDialog(version, parent);
}

QString ThemeFileModel::showModelSelecionDialog(const QString &version, QWidget *parent) {
  SelectModelDialog * dlg = new SelectModelDialog(version, parent);
  int value = dlg->exec();
  if (value == QDialog::Accepted) {
    return dlg->getSelectedModel();
  }
  return QString::null;
}

void ThemeFileModel::extract(const QString &_filename) {
  // TODO call themefilereader with &stream
  KTar *themefile = new KTar(_filename, "application/x-tar");
  themefile->open(IO_ReadOnly);
  
  /* create temporary dir */
  if(temp_dir) {
    delete temp_dir;
    temp_dir = 0;
  }
  temp_dir = new KTempDir(locateLocal("tmp", "themekreator"));
  temp_dir->setAutoDelete(true);
  
  if( temp_dir->status() != 0)
  {
     KMessageBox::sorry(0, i18n("Temporary files could not be created. Aborting."));
     delete themefile;
     //return QString::null;// THEMEKREATOR_RETRY;
  }

  this->_extract(themefile->directory(), temp_dir->name());
  themefile->close();
  delete themefile;
  dir = temp_dir->name();
}

void ThemeFileModel::_extract(const KArchiveDirectory *theme_dir, const QString &temp_dir) {
  //TODO: check this
  QStringList entries = theme_dir->entries();
  for ( QStringList::Iterator it = entries.begin(); it != entries.end(); ++it ) {
    if ( theme_dir->entry(*it)->isFile () ) {
      KArchiveFile *entry = (KArchiveFile *)theme_dir->entry(*it);
      entry->copyTo(temp_dir);
    } else { // it's a directory
      _extract((KArchiveDirectory *)theme_dir->entry(*it), temp_dir);
    }
  }
}

QString ThemeFileModel::getXMLFile() {
  if (temp_xmlfile == QString::null) {
    if (temp_dir) {
      QDir temp_qdir = QDir( temp_dir->name() );
      temp_qdir.setNameFilter("*.xml");
      temp_qdir.setFilter( QDir::Files | QDir::Hidden | QDir::NoSymLinks );
      temp_qdir.setSorting( QDir::Name );
      temp_xmlfile = temp_qdir.entryList()[0];
    }
  }
  return temp_xmlfile;
}

int ThemeFileModel::readXML() {
  QString filename = getXMLFile();
  if ( filename.isEmpty() ) {
    return -1;
  }
  filename = temp_dir->name() + filename;

  if( filename.length() == 0) return -1;
  
  QFile xmlfile( filename );

  if(!xmlfile.open(IO_ReadOnly))
  {
     //KMessageBox::sorry(0, i18n("Cannot open xml file."));
     return -1;
  }
  QTextStream xmlstream(&xmlfile);
  QString data = xmlstream.read();
  // filter any comments before the root tag (this is necessary for the dom parser trick below)
  QRegExp exp_1("<?xml version=\"1.0\"?>?*<Sony", true, true);
  exp_1.setMinimal( TRUE );
  data.replace(exp_1, "<?xml version=\"1.0\"?>\n<Sony");
  QRegExp exp_2("<?xml version=\"1.0\"?>?*<Eric", true, true);
  exp_2.setMinimal( TRUE );
  data.replace(exp_2, "<?xml version=\"1.0\"?>\n<Eric");
  // just a trick for our dom parsaer
  data.replace("<!--", "<comment>");
  data.replace("-->", "</comment>");
  
  m_domdoc.setContent( data );
  //this->filename = filename;
  return 0;
}

void ThemeFileModel::loadData() {
  QDictIterator<ThemeAttribute> it( *this->getAttributes() );
  for( ; it.current(); ++it ) {
    QString value = getDOMValue(it.current()->get_xmlName(), it.current()->getXMLTagAttribute());
    int i = value.findRev( QDir::separator() );
    i++;
/*    if ( i == -1 ) {
      i = 0;
    }*/
    value = value.right(value.length() - i);
    if ( ! value.isEmpty() ) {
      it.current()->set_value(value);
    }
  }
 
//   if ( getDOMValue("Theme_name").isEmpty() ) {
//     this->getAttribute("Theme_name")->set_value( i18n("unknown") );
//   }
//   else {
//     this->getAttribute("Theme_name")->set_value( getDOMValue("Theme_name") );
//   }
//   this->getAttribute("Theme_creator")->set_value( getDOMValue("Theme_creator") );
//   this->getAttribute("Theme_created")->set_value( getDOMValue("Theme_created") );
//   this->getAttribute("Theme_modified_by")->set_value( getDOMValue("Theme_modified_by") );

  this->getAttribute("Author_firstname")->set_value( getDOMValue("Author_firstname", "value") );
  this->getAttribute("Author_lastname")->set_value( getDOMValue("Author_lastname", "value") );
  this->getAttribute("Author_organisation")->set_value( getDOMValue("Author_organisation", "value") );
  this->getAttribute("Author_email")->set_value( getDOMValue("Author_email", "value") );
  this->getAttribute("Author_url")->set_value( getDOMValue("Author_url", "value") );



  //emit valueChanged();
  emit dirChanged(this->dir);
  this->setModified(false);
}

QString ThemeFileModel::getDOMValue(QString name, QString att) {
  QDomNodeList nl;
  QDomNamedNodeMap nnm;
  nl = m_domdoc.elementsByTagName(name);
  if (nl.count() > 0) {
     nnm = nl.item(0).attributes();
     if (att == "Color")
       return nnm.namedItem(att).toAttr().value().mid(2,6);
     else
       return nnm.namedItem(att).toAttr().value();
  }
  return "";
}

void ThemeFileModel::slotThemeNameChanged() {
  if ( lvi_xmlfilename != 0 ) {
/*    if ( this->getAttribute("Theme_name")->get_value().isEmpty() ) {
      lvi_xmlfilename->setText(0, i18n("unknown.xml") );
      lvi_xmlfilename->setVisible(true);
    }
    else {
      lvi_xmlfilename->setText(0, this->getAttribute("Theme_name")->get_value() + ".xml" );
      lvi_xmlfilename->setVisible(true);
    }*/
  }
}

void ThemeFileModel::updateSettings() {
  QDictIterator<ThemeAttribute> it( attributes );
  ThemeAttribute *attr ;
  for( ; it.current(); ++it ) {
    attr = (ThemeAttribute *) it.current();
    attr->updateSettings();
  }

}


void ThemeFileModel::slot_attributeViolation( ThemeChecker::Violation , ThemeAttribute* , ThemeAttribute* ) {
  //cout << attribute_1->get_xmlName() << "<-->" << attribute_2->get_xmlName() << endl;
/*  if ( violation == ThemeChecker::VIOL_COLOR_DISTANCE ) {
    attribute_1->set_invalid( true );
    attribute_2->set_invalid( true );
  }*/
}

#include "themefilemodel.moc"
