/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef THEMEFILEMODEL_H
#define THEMEFILEMODEL_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "themechecker.h"
#include "themeattribute.h"
#include "previewdatamodel.h"

#include <qobject.h>
#include <qstringlist.h> 
#include <qdict.h>
#include <qmap.h>
#include <qdom.h>
#include <math.h>
#include <karchive.h> 
#include <kurl.h>

class QSize;
class QStringList;
class KListView;
class QListViewItem;
class KTempDir;
class QWidget;

/**
@author Matthias Wetzka
*/
class ThemeFileModel : public QObject
{
Q_OBJECT
public:
    ThemeFileModel();
    ~ThemeFileModel();
    
    void setThemeVersion(QString &version);
    void setImageFormats(QString &formats);
    void setAttributes(const QDict<ThemeAttribute> &attributes);
    
    QString getAttributeValue(QString attribute);
    //void setAttributeValue(QString attribute, QString value);
    
    ThemeAttribute *getAttribute(QString attribute);
    
    QDict<ThemeAttribute> *getAttributes();

    void createListViewItems(KListView * listView);

    bool isModified();
    void setModified(bool);
    bool isInvalid();
    void setInvalid(bool);
    
    QString getModelFromVersion(QWidget *parent = 0);
    
    
    QString getVersion() { return this->m_version; };
    QString getImageFormats() { return this->m_imageformats; };

    void setPreviews(QDict<PreviewDataModel> *previews) { this->m_previews = previews; };
    QDict<PreviewDataModel> * getPreviews() { return this->m_previews; };
    PreviewDataModel * getPreview(const QString &name) { return this->m_previews->find(name); };
    void setPreviewNames(const QStringList &preview_names) { this->m_preview_names = preview_names; }
    QStringList getPreviewNames() { return this->m_preview_names; }
    
    //QStringList file_list;
    QString dir;

    void setURL( const KURL & );
    KURL getURL();

    /*QString
      model, 
      themeName, 
      creator, 
      creationDate, 
      modifier;*/

   QString asXML();
   void reset();
      
 public slots:
    void slot_attributeViolation( ThemeChecker::Violation, ThemeAttribute*, ThemeAttribute* );
    /*void setModel(const QString &data) { this->model = data; } //emit valueChanged(); }
    void setThemeName(const QString &data) { this->themeName = data; } //emit valueChanged(); }
    void setCreator(const QString &data) { this->creator = data; } //emit valueChanged(); }
    void setCreationDate(const QString &data) { this->creationDate = data; } //emit valueChanged(); }
    void setModifier(const QString &data) { this->modifier = data; } //emit valueChanged(); }*/
  
    void extract( const QString & thmfilename);
    QString getETTModel(QWidget *parent = 0);
    void loadData();
    int readXML();
    void handleModified();
    void unsetModified();
    void slotThemeNameChanged();
    void updateSettings();
       
 signals:
    void valueChanged();
    void dirChanged(const QString &);
    void modifiedChanged(bool);
    void invalidChanged(bool);
    
 private:
   bool modified;
   bool invalid;
   KURL m_url;
   QListViewItem * lvi_xmlfilename;
   KTempDir * temp_dir;
   QString temp_xmlfile;
   QString m_version;
   QString m_imageformats;
   QString asTag(QString, QString, QString);
   QString showModelSelecionDialog(const QString &version, QWidget *parent = 0);
   QString getXMLFile();
   QDomDocument m_domdoc;
   QDict<ThemeAttribute> attributes;
   QMap<QString, QListViewItem*> file_list;
   QDict<PreviewDataModel> *m_previews;
   QStringList m_preview_names;
   
   QString getDOMValue(QString name, QString  att = "Name");
   void _extract(const KArchiveDirectory *, const QString &);
};

#endif
