/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "uigroup.h"

#include <qwidget.h>
#include <qgroupbox.h>
#include <qlayout.h>

#include <kdialog.h>

UIGroup::UIGroup(QString name, int strips)
 : QPtrList<ThemeAttribute>()
{
  this->name = name;
  this->strips = strips;
 }


UIGroup::~UIGroup()
{
/*  ThemeAttribute * attribute;
  for ( attribute = first(); attribute; attribute = next() ) {
    delete attribute;
  }*/
}


QWidget *UIGroup::widget(QWidget * parent) {
  QGroupBox *box = new QGroupBox(0, Qt::Vertical, name, parent, "groupbox"+name);
  box->layout()->setSpacing( KDialog::spacingHint() );
  box->layout()->setMargin( KDialog::marginHint() );
  
  QGridLayout * layout = new QGridLayout( box->layout() );
  layout->setAlignment( Qt::AlignTop );

  int i = 0;
  int my_strips = strips + (strips/2 - 1);
  if( my_strips > 2) {
    for( int i = 0; i < my_strips; i++ ) {
      if ( (i%my_strips)%3 == 0  )
        layout->setColStretch ( i, 1 );
      else if ( (i%my_strips)%3 == 1  )
        layout->setColStretch ( i, 0 );
      else
        layout->setColStretch ( i, 0 );
    }
  } else {
    layout->setColStretch ( 0, 0 );
    layout->setColStretch ( 1, 1 );
  }
  ThemeAttribute * attribute;
  QWidget * labelWidget;
  QWidget * editWidget;
  QSpacerItem * spacerItem;
  for ( attribute = first(); attribute; attribute = next() ) {
    labelWidget = attribute->createLabelWidget(box);
    layout->addWidget(labelWidget, i/my_strips, i%my_strips);
    i++;
    editWidget = attribute->createEditWidget(box);
    layout->addWidget(editWidget, i/my_strips, i%my_strips);
    i++;
    if( (i%my_strips) % 3 == 2 ) {
      spacerItem = new QSpacerItem ( 12, 0, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum );
      layout->addItem(spacerItem, i/my_strips, i%my_strips);
      i++;
    }
  }
  
  return box;
}
