/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "themekreator.h"
#include "config/themekreatorconfig.h"
#include "config/settingsdialog.h"
#include "model/themefilemodel.h"
#include "themechecker.h"
#include "model/themeattributeimage.h"
#include "model/uimodel.h"
#include "model/themedefinitionloader.h"
#include "ui/mycolorrequester.h"
#include "ui/mylineedit.h"
#include "ui/themeviewwidget.h"
#include "ui/selectmodelwidget.h"
#include "ui/newmodeldialog.h"
#include "ui/phonepreviewdialog.h"

#include <qlabel.h>
#include <qcursor.h>
#include <qtabwidget.h>

#include <kcommand.h>
#include <kapplication.h>
#include <kmainwindow.h>
#include <klocale.h>
#include <kstddirs.h>
#include <kstdaction.h>
#include <ksavefile.h>
#include <kaction.h>
#include <kiconloader.h>
#include <kstatusbar.h> 
#include <kurlrequester.h> 
#include <kmessagebox.h> 
#include <kio/netaccess.h>
#include <kfiledialog.h> 
#include <karchive.h> 
#include <ktar.h>
#include <ktempfile.h> 
#include <ktempdir.h> 
#include <klistview.h>
#include <kconfigdialog.h>
#include <kwin.h>
#include <ksqueezedtextlabel.h>

// #include <libkbluetooth/deviceaddress.h>
// #include <libkobex/kobexclient.h>


ThemeKreator::ThemeKreator()
    : KMainWindow( 0, "themekreator#" )
{
    // set the shell's ui resource file
    setXMLFile("themekreatorui.rc");

    KWin::setIcons( winId(), kapp->icon(), kapp->miniIcon() );

    theme_model = new ThemeFileModel();
    connect(theme_model, SIGNAL(modifiedChanged(bool)), this, SLOT(updateCaption()) );
    connect(theme_model, SIGNAL(invalidChanged(bool)), this, SLOT(updateStatusBar()) );
    
    theme_checker = new ThemeChecker(theme_model);
    connect(theme_checker, SIGNAL(signal_attributeViolation( ThemeChecker::Violation, ThemeAttribute*, ThemeAttribute* )), theme_model, SLOT(slot_attributeViolation( ThemeChecker::Violation, ThemeAttribute*, ThemeAttribute* )));

    m_view = new ThemeViewWidget(this, "themeview");
    m_view->setFileModel(theme_model);
    //new KListViewItem(m_view->lvThemeContents, "foo", QString("???"), QString("???"), QString::null, i18n("Theme description"));
    commandHistory = new KCommandHistory(actionCollection(), true);
    // when all undos are done, theme is not modified
    connect(commandHistory, SIGNAL(documentRestored()), theme_model, SLOT(unsetModified()));
    
    //KWin::setIcons(app.winID(), 
   
    //this->setFileModel(new ThemeFileModel(commandHistory));
    
    //m_view->setUIModel(new UIModel() );
    m_themedefinitionloader = 0;
    m_view->tab->setEnabled(false);

    setCentralWidget( m_view );
    
    setupStatusBar();
    setupActions();
    readSettings();
    adjustSize();
    setAutoSaveSettings();
    
   /* // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));*/
}

ThemeKreator::~ThemeKreator()
{
  if ( theme_model != 0 ) { delete theme_model; }
  if ( m_themedefinitionloader != 0 ) { delete m_themedefinitionloader; }
  if ( commandHistory  != 0 ) { delete commandHistory; }
}

void ThemeKreator::setFileModel(ThemeFileModel *filemodel) {
  theme_model = filemodel;
  m_view->setFileModel(theme_model);
//  theme_model->createListViewItems(m_view->lvThemeContents);
  //commandHistory->clear();
}

void ThemeKreator::setupStatusBar()
{
  statusBar()->insertItem("", ID_MESSAGE, 10 );
  statusBar()->insertFixedItem("", ID_PHONE_MODEL, true);
  
  statusBar()->setItemAlignment( ID_MESSAGE, AlignLeft|AlignVCenter );
  statusBar()->setItemAlignment( ID_PHONE_MODEL, AlignLeft|AlignVCenter );

  statusBar()->setItemFixed(ID_PHONE_MODEL, statusBar()->fontMetrics().width("SonyEricsson X888X"));
}

void ThemeKreator::setupActions() 
{

    KStdAction::quit(this, SLOT(close()), actionCollection());
    //KStdAction::openNew(this, SLOT(newtheme()), actionCollection());
    KStdAction::open(this, SLOT(open()), actionCollection());
    recent = KStdAction::openRecent(this, SLOT(openRecent( const KURL &)), actionCollection());
    
    KStdAction::openNew(this, SLOT(openNew()), actionCollection());
    KStdAction::save(this, SLOT(save()), actionCollection());
    KStdAction::saveAs(this, SLOT(saveAs()), actionCollection());
    //KStdAction::showStatusbar(this, SLOT(toggleStatusBar()), actionCollection());
    KStdAction::preferences(this, SLOT(showSettings()), actionCollection());
    action_show_preview_dialog = new KAction(i18n("Show &preview window"), QIconSet(BarIcon("view_icon")), 0, this, SLOT(showPreview()), actionCollection(), "show_preview_wnd");
    action_show_preview_dialog->setEnabled(false);
    //(void)new KAction(i18n("Send theme to &phone"), QIconSet(BarIcon("mail_send")), 0, this, SLOT(sendThemeToPhone()), actionCollection(), "sendfile");
    
    /*(void)new KAction(i18n("&Properties"), BarIcon("edit"), 0, this, SLOT(theme_properties()), actionCollection(), "theme_prop");*/


    setupGUI();
}


bool ThemeKreator::queryExit( void ) {
  // save recent files menu
  KConfig *config = kapp->config();
  recent->saveEntries( config );
  config->sync();

  return true;
}

bool ThemeKreator::queryClose( void ) {
  if ( ! theme_model->isModified() ) {
    queryExit();
    return true;
  }
  
  QString msg = i18n("This theme has been modified.\nWould you like to save it?" );
  switch( KMessageBox::warningYesNoCancel( this, msg, QString::null,
                       KStdGuiItem::save(), KStdGuiItem::discard() ) )
  {
     case KMessageBox::Yes: // Save, then exit
       this->save();
       if ( theme_model->isModified() ) {
         return false; // Still modified? Don't exit!
       }
     case KMessageBox::No: // Don't save but exit.
          return true;
     case KMessageBox::Cancel: // Don't save and don't exit.
     default:
          return false; // Don't exit...
  }
  return true; // Exit.
}

void ThemeKreator::toggleStatusBar() {
  if(statusBar()->isVisible())
    statusBar()->hide();
  else
    statusBar()->show();
}
   
void ThemeKreator::toggleToolBar() {
  if(toolBar()->isVisible())
    toolBar()->hide();
  else
    toolBar()->show();
}

void ThemeKreator::open() 
{
/*  if ( !theme_model->getFilename().isEmpty() || theme_model->isModified() ) {
    ThemeKreator *t = new ThemeKreator();
    t->open();
    t->show();
    return;
  }*/
  ThemeKreator *instance = 0;

  while( 1 )
  {
    KURL url = KFileDialog::getOpenURL(
      ":themedir",
      "*.thm|"+i18n("Theme files (*.thm)")+"\n*|"+i18n("All files"), 
      this, 
      i18n("Choose a theme") 
    );
    if( url.isEmpty() )
    {
      return;
    }

    if ( !theme_model->getVersion().isEmpty() ) {
//    if ( !theme_model->getFilename().isEmpty() || theme_model->isModified() ) {
      instance = new ThemeKreator();
      if ( instance == 0) {
        return;
      }
    }
    else {
      instance = this;
    }
    
    KIO::UDSEntry entry;
    KIO::NetAccess::stat(url, entry, this);
  
    QString tmpfile;
    KIO::NetAccess::download( url, tmpfile, this );
  
    instance->setCursor(QCursor(Qt::WaitCursor));
    instance->m_view->tab->setCurrentPage(0);
    instance->theme_model->reset();
//    theme_model->setSignalsEnabled(false);
    int result = instance->openFile( tmpfile, 0);
    if( result == THEMEKREATOR_OK )
    {
      KIO::NetAccess::removeTempFile( tmpfile );
      instance->setCursor(QCursor(Qt::ArrowCursor));
      instance->recent->addURL( url );
      instance->theme_model->setURL( url );
      break;
    }
    else
    {
      KIO::NetAccess::removeTempFile( tmpfile );
      instance->setCursor(QCursor(Qt::ArrowCursor));
      if( instance != this ) { delete instance; }
      continue;
    }
    //theme_model->setSignalsEnabled(true);
  }
  instance->show();
  instance->theme_model->setModified( false );
  instance->updateStatusBar();

  instance->updateCaption();
}

void ThemeKreator::openRecent( const KURL & url) {
  if ( !theme_model->getVersion().isEmpty() ) {
    ThemeKreator *t = new ThemeKreator();
    t->openURL( url, 0);
    t->show();
  }
  else {
    openURL( url, 0);
  }
}

void ThemeKreator::openURL( const KURL& _url, int _mode )
{
  if ( !_url.isValid() )
  {
      QString string;
      string = i18n( "Malformed URL: \n%1").arg(_url.prettyURL());

      KMessageBox::sorry(this, string);
      return;
  }

  QString tmpFile;
  int result = THEMEKREATOR_OK;
  if (KIO::NetAccess::download(_url, tmpFile, this))
  {
      m_view->tab->setCurrentPage(0);
      theme_model->reset();
      this->setCursor(QCursor(Qt::WaitCursor));
//        theme_model->setSignalsEnabled(false);
      result = openFile(tmpFile, _mode);

//        theme_model->setSignalsEnabled(true);
      this->setCursor(QCursor(Qt::ArrowCursor));
      KIO::NetAccess::removeTempFile( tmpFile );
  }
  else
  {
    //if (_mode & OPEN_NEW == 0)
    {
      KMessageBox::error(this, KIO::NetAccess::lastErrorString()+"fgoobae" );
      return;
    }
  }
  if (result == THEMEKREATOR_OK)
  {
      //m_url = _url;
      //setFileCaption();
      recent->addURL( _url );
      theme_model->setURL( _url );
      commandHistory->clear();
      //eframe->setModified(false);
      //setGeneralStatusField(i18n("Done"));
  }
  theme_model->setModified( false );
  updateStatusBar();
  updateCaption();
}


  
int ThemeKreator::openFile(QString _filename, int _mode )
{
  if (_filename == QString::null) return THEMEKREATOR_RETRY;
  if ( _filename.startsWith("file:") ) {
    _filename.remove(0, 5);
  }
  QFileInfo info(_filename);

  if(info.isDir())
  {
     KMessageBox::sorry(this, i18n("You have specified a folder"));
     return THEMEKREATOR_RETRY;
  }

  if(!info.exists() || !info.isFile())
  {
     KMessageBox::sorry(this, i18n("The specified file does not exist"));
     return THEMEKREATOR_RETRY;
  }

  QFile file(_filename);

  if(!file.open(IO_ReadOnly))
  {
     KMessageBox::sorry(this, i18n("You do not have read permission to this file."));
     return THEMEKREATOR_RETRY;
  }

  
  theme_model->reset();
  theme_model->extract(_filename);
  if ( theme_model->readXML() < 0 ) {
    KMessageBox::error(this, i18n("Could not locate the XML file within the themefile."));
    return THEMEKREATOR_RETRY;
  }
  QString model = theme_model->getETTModel(this);
  if (model == QString::null) {
    //this->deleteLater();
    return THEMEKREATOR_ABORT;
  }
  else if (model == "unknown") {
    model = theme_model->getModelFromVersion(this);
    if (model == QString::null) {
      //this->deleteLater();
      //KMessageBox::error(this, i18n("Could not determine the phone model."));
      return THEMEKREATOR_ABORT;
    }
    else if (model == "unknown") {
      KMessageBox::sorry(this, i18n("This version of ThemeKreator can't understand this theme file."));
      return THEMEKREATOR_RETRY;
    }
  }
  
  m_view->tab->setEnabled(true);
  action_show_preview_dialog->setEnabled(true);
  m_themedefinitionloader = new ThemeDefinitionLoader(this, this->m_view); //mainWin, mainWin->m_view);
  m_themedefinitionloader->load( model );
  
  m_view->tab->setCurrentPage(0);
  theme_model->loadData();
  
  //theme_model->getAttribute("Phone_model")->set_value( model );
//   if ( ! theme_model->getAttribute("Theme_creator")->get_value().isEmpty() ) {
//     m_view->leCreator->setEnabled( false );
//   }
//   else {
//     m_view->leCreator->setEnabled( true );
//   }

  theme_model->createListViewItems(m_view->lvThemeContents);
  //theme_model->setFile_list( files_list );
  
  m_view->refreshController(); 
  
  commandHistory->clear();  
  theme_model->setModified( false );
  updateStatusBar();
  updateCaption();
  
  return THEMEKREATOR_OK;
}
  

void ThemeKreator::save() {
  if (!theme_model->getURL().isValid())
  {
     saveAs();
     return;
  }

  int result = THEMEKREATOR_OK;

  result =  saveURL(theme_model->getURL()); // error messages are handled by saveFile

  if ( result == THEMEKREATOR_OK ){
  }
}

void ThemeKreator::saveAs() {
  KURL url;
  while(true)
  {
     url = KFileDialog::getSaveURL(
        theme_model->getURL().isValid() ? ":themedir" : theme_model->getURL().url(),
        "*.thm|"+i18n("Theme files (*.thm)")+"\n*|"+i18n("All files"), 
        this, 
        i18n("Choose a filename") );

     if (url.isEmpty()) {
        return;
     }

     if ( KIO::NetAccess::exists(url, false, this) )
     {
        int result = KMessageBox::warningContinueCancel( this,
           i18n( "A file named \"%1\" already exists. "
                 "Are you sure you want to overwrite it?" ).arg( url.prettyURL() ),
           i18n( "Overwrite File?" ),
           i18n( "Overwrite" ) );

        if (result != KMessageBox::Continue)
           continue;
     }
     break;
  }

  int result = saveURL(url); // error messages are handled by saveFile
  
  if ( result == THEMEKREATOR_OK ) {
    recent->addURL( url );
    theme_model->setURL( url );
    theme_model->setModified( false );
    updateStatusBar();
    updateCaption();
  }

/*  QString s = KFileDialog::getSaveFileName(
        theme_model->getFilename().isEmpty() ? ":themedir" : theme_model->getFilename(),
        "*.thm|"+i18n("Theme files (*.thm)")+"\n*|"+i18n("All files"), 
        this, 
        i18n("Choose a filename") );
  if (s.length() > 0) {
    theme_model->setFilename( s );
    save();
    recent->addURL( KURL(s) );
  }*/
}

int ThemeKreator::saveURL( const KURL& _url )
{
  if ( theme_model->isInvalid() ) {
    QString msg = i18n("This theme contains errors. You can save this theme now, but if you upload it to the phone without correcting the errors, it will probably not work as expected. Do you still want to save the theme now?" );
    switch( KMessageBox::warningYesNo( this, msg, QString::null, KStdGuiItem::yes(), KStdGuiItem::no() ) )
    {
      case KMessageBox::Yes:
        break;
      case KMessageBox::No:
      default:
        return THEMEKREATOR_ABORT;
    }
  }
    
  if ( !_url.isValid() )
  {
    KMessageBox::sorry(this, i18n("Malformed URL"));
    return THEMEKREATOR_RETRY;
  }

  // Just a usual file ?
  if ( _url.isLocalFile() )
  {
    return saveFile( _url.path(), true );
  }

  KTempFile tmpFile;
  tmpFile.setAutoDelete(true);
  //theme_model->setModified( true );
  saveFile( tmpFile.name(), false );

  if (KIO::NetAccess::upload( tmpFile.name(), _url, this ) == false)
  {
    KMessageBox::error(this, "Could not save remote file");
    return THEMEKREATOR_RETRY;
  }

  return THEMEKREATOR_OK;
}

int ThemeKreator::saveFile( const QString& _filename, bool backup )
{
  QFileInfo info(_filename);
  if(info.isDir())
  {
     KMessageBox::sorry(this, i18n("You have specified a folder"));
     return THEMEKREATOR_RETRY;
  }

  if (backup && ThemeKreatorConfig::backupCopy() && QFile::exists(_filename))
  {
     if (!KSaveFile::backupFile(_filename))
     {
        KMessageBox::sorry(this, i18n("Unable to make a backup of the original file."));
     }
  }

  KTar archive = KTar( _filename, "application/x-ustar");
  if(!archive.open(IO_WriteOnly))
  {
     KMessageBox::sorry(this, i18n("Unable to write to file."));
     return THEMEKREATOR_RETRY;
  }
  
  QDictIterator<ThemeAttribute> it( *(theme_model->getAttributes()) );
  ThemeAttributeImage * image;
  QStringList added_files;
  QFileInfo file_info;
  for( ; it.current(); ++it ) {
    //it.current()->get_value()
    if (it.current()->get_type() == ThemeAttribute::Image) {
      if (it.current()->get_value().length() > 0) {
        image = (ThemeAttributeImage *) it.current();
        if ( ! added_files.contains( image->get_value() ) ) {
          QString filename = image->get_directory()+image->get_value();
          file_info.setFile( filename );
          if ( file_info.isSymLink() ) {
            filename = file_info.readLink();
          }
          archive.addLocalFile(filename, image->get_value());
          added_files.append( image->get_value() );
        }
      }
    }
  }

  /* write xml */
  KTempFile xmlfile = KTempFile();
  xmlfile.setAutoDelete(false);
  QTextStream *stream = xmlfile.textStream();
  *stream << theme_model->asXML();
  xmlfile.close();
  archive.addLocalFile(xmlfile.name(), "theme.xml");    
  archive.close();
/*  if(archive.status())
  {
     KMessageBox::sorry(this, i18n("Could not save file."));
     return THEMEKREATOR_RETRY;
  }*/
  xmlfile.unlink();
  commandHistory->clear();
  theme_model->setModified(false);
  updateCaption();
  return THEMEKREATOR_OK;

/*
  QFileInfo info(_filename);
  bool bSoftWrap = (Prefs::wrapMode() == Prefs::EnumWrapMode::SoftWrap);

  if(info.isDir())
  {
     KMessageBox::sorry(this, i18n("You have specified a folder"));
     return KEDIT_RETRY;
  }
  if(file.status())
  {
     KMessageBox::sorry(this, i18n("Could not save file."));
     return KEDIT_RETRY;
  }

  if (backup && Prefs::backupCopies() && QFile::exists(_filename))
  {
     if (!KSaveFile::backupFile(_filename))
     {
        KMessageBox::sorry(this, i18n("Unable to make a backup of the original file."));
     }
  }

  // WABA: We don't use KSaveFile because it doesn't preserve hard/soft
  // links when saving. Most applications don't care about this, but an
  // editor is supposed to preserve such things.

  QFile file(_filename);
  if(!file.open(IO_WriteOnly))
  {
     KMessageBox::sorry(this, i18n("Unable to write to file."));
     return KEDIT_RETRY;
  }

  QTextStream textStream(&file);
  QTextCodec *codec;
  if (!encoding.isEmpty())
    codec = QTextCodec::codecForName(encoding.latin1());
  else
    codec = QTextCodec::codecForLocale();
  textStream.setCodec(codec);

  eframe->saveText( &textStream, bSoftWrap );
  file.close();

  if(file.status())
  {
     KMessageBox::sorry(this, i18n("Could not save file."));
     return KEDIT_RETRY;
  }
  eframe->setModified(false);
  return KEDIT_OK;*/
}




int ThemeKreator::selectModel() {
  NewModelDialog * dlg = new NewModelDialog(m_view);
  int value = dlg->exec();
  if (value == QDialog::Accepted) {
    m_themedefinitionloader = new ThemeDefinitionLoader(this, this->m_view); //mainWin, mainWin->m_view);
    m_view->tab->setEnabled(true);
    action_show_preview_dialog->setEnabled(true);
    m_themedefinitionloader->load( dlg->getSelectedModel() );
    theme_model->createListViewItems(m_view->lvThemeContents);
    m_view->tab->setCurrentPage(0);
    theme_model->setModified( false );
    updateStatusBar();
    updateCaption();
    return THEMEKREATOR_OK;
/*    switch (dlg->getTabWidget()->currentPageIndex ()) {
      case 0: {
        m_themedefinitionloader = new ThemeDefinitionLoader(this, this->m_view); //mainWin, mainWin->m_view);
        m_themedefinitionloader->load( dlg->getSelectedModel() );
        theme_model->setModified( false );
        updateCaption();
        break;
      }
      case 1: {
        this->openURL( dlg->getSelectedURL(), 0 );
        break;
      }
      case 2: {
        this->openURL( dlg->getSelectedRecent(), 0 );
        //this->openFile( dlg->getSelectedRecent(), 0 );
        break;
      }
    }*/
  }
  else {
    return THEMEKREATOR_ABORT;
    //this->close();
    //this->deleteLater();
  }
}

void ThemeKreator::openNew() {
//  if ( !theme_model->getFilename().isEmpty() || theme_model->isModified() ) {
  if ( !theme_model->getVersion().isEmpty() ) {
    ThemeKreator *t = new ThemeKreator();
    int result = t->selectModel();
    if( result == THEMEKREATOR_ABORT ) {
      t->close();
      t->deleteLater();
    }
    else {
      t->show();
    }
    return;
  }
  else {
    selectModel();
  }
}

void ThemeKreator::sendThemeToPhone() {
 /* KTempDir *temp_dir = new KTempDir(locateLocal("tmp", "themekreator"));
  temp_dir->setAutoDelete(true);
  

     
   KProcess *proc = new KProcess;
   *proc << "kbtobexclient";
   *proc << "/home/metty/temp/screenshot1.png";*/
/*   QApplication::connect(proc, SIGNAL(processExited(KProcess *)),
                         pointer_to_my_object, SLOT(my_objects_slot(KProcess *)));*/
/*   proc->start();

   delete temp_dir;*/
   
   // ------------ intended for later use --------------------
    
  /* copy source form kbtobexclient for sending files */
//   KBluetooth::DeviceAddress bda = KBluetooth::DeviceAddress("00:0E:07:81:B4:BD");
//  ObexClient * client = new ObexClient( bda, 6, this);
  
//  client->startConnection();

  // Setup Progress bar
  //bar->setProgress(0);

  // Start client
  //client->sendFile( );
  
//  client->stopConnection();

/*  if(!client->connected())
    client->startConnection();

  // Setup Progress bar
  bar->setProgress(0);

  // Start client
  client->sendFile(url.path());*/
}

void ThemeKreator::showPreview() {
//   if ( KConfigDialog::showDialog( "preview" ) )
//     return;

  PhonePreviewDialog * dialog = new PhonePreviewDialog(theme_model, this, "preview");

/*  //User edited the configuration - update your local copies of the
  //configuration data 
  connect( dialog, SIGNAL(settingsChanged()), this, SLOT(updateSettings()) );*/
 
  dialog->show();
  
}


// void ThemeKreator::saveProperties()
// {
//   //ThemeKreatorConfig::setServerName( m_server );
//   //ThemeKreatorConfig::setPort( m_port );
//   ThemeKreatorConfig::writeConfig();
// }
//   
// void ThemeKreator::readProperties()
// {
//   // config will have been opened for you, just
//   // read what you saved in saveProperties(..)
//   // and recover your program.
// }


void ThemeKreator::updateSettings()
{
  readSettings();
  //setupEditWidget();
  theme_model->updateSettings();
}

void ThemeKreator::readSettings( void )
{
  recent->loadEntries( kapp->config() );
}

void ThemeKreator::showSettings() {
  if ( KConfigDialog::showDialog( "settings" ) )
    return;

   //KConfigDialog didn't find an instance of this dialog, so lets create it :
  KConfigDialog* dialog = new SettingsDialog( this, "settings", ThemeKreatorConfig::self() );

  //User edited the configuration - update your local copies of the
  //configuration data 
  connect( dialog, SIGNAL(settingsChanged()), this, SLOT(updateSettings()) );
 
  dialog->show();
}

void ThemeKreator::updateCaption() {
  if ( theme_model->getURL().isEmpty() ) {
    this->setCaption(i18n("New Theme"), theme_model->isModified() );
  }
  else {
    this->setCaption(theme_model->getURL().fileName(), theme_model->isModified() );
  }
}


void ThemeKreator::updateStatusBar() {
  if ( theme_model->isInvalid() ) {
    statusBar()->changeItem( i18n("Warning: the theme contains errors!"), ID_MESSAGE );
  }
  else {
    statusBar()->changeItem( "", ID_MESSAGE );
  }
  statusBar()->changeItem(i18n("SonyEricsson") + " " + theme_model->getAttributeValue("Phone_model"), ID_PHONE_MODEL );
}


#include "themekreator.moc"
