/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _THEMEKREATOR_H_
#define _THEMEKREATOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qstring.h>
#include <kmainwindow.h>
#include <kactionclasses.h> 

#define THEMEKREATOR_OK 0
#define THEMEKREATOR_RETRY 1
#define THEMEKREATOR_ABORT 2

// StatusBar field IDs
#define ID_MESSAGE 1
#define ID_PHONE_MODEL 2

class ThemeViewWidget;
class KCommandHistory;
class ThemeDefinitionLoader;
class ThemeFileModel;
class ThemeChecker;
class KAction;
class KSqueezedTextLabel;


/**
 * @short Application Main Window
 * @author Matthias Wetzka <matthias.wetzka@gmx.de>
 * @version 0.1
 */
class ThemeKreator : public KMainWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    ThemeViewWidget *m_view;
    KCommandHistory *commandHistory;
    ThemeKreator();

    void openURL( const KURL& _url, int _mode );
    int saveURL( const KURL& );
    
    void toggleStatusBar();
    void toggleToolBar();
    int selectModel();
    
    void setFileModel(ThemeFileModel *filemodel);
    ThemeFileModel * getFileModel() { return this->theme_model; }
    ThemeChecker * getThemeChecker() { return this->theme_checker; }

    /**
     * Default Destructor
     */
    virtual ~ThemeKreator();
public slots:
    void open();
    void openNew();
    void openRecent( const KURL &);
    void save();
    void saveAs();
    void showPreview();
    void sendThemeToPhone();
    void updateCaption();
    void updateStatusBar();
    
protected:
    bool queryExit( void );
    bool queryClose( void );
    void setupActions();
    void setupStatusBar();



    
private slots:
    void updateSettings();
    void readSettings();
    void showSettings();





    
private:
    ThemeDefinitionLoader * m_themedefinitionloader;
    ThemeFileModel *theme_model;
    ThemeChecker *theme_checker;
    KRecentFilesAction *recent;
    int openFile( QString _filename, int _mode/*, bool _undoAction*/ );
    int saveFile( const QString& , bool backup=false );
    KSqueezedTextLabel *m_statusLabel;
    KAction* action_show_preview_dialog;
};

#endif // _THEMEKREATOR_H_
