//
// C++ Implementation: animchecker
//
// Description: 
//
//
// Author: Matthias Wetzka <matthias.wetzka@gmx.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "animchecker.h"

#include <qstring.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qcstring.h>

int AnimChecker::isAnimatedGIF(const QString &path) {
    struct GIFLSDstruct { //Logical Screen Descriptor
        unsigned short width; 
        unsigned short height;
        unsigned char packed_fields;
//              <Packed Fields>  =      Global Color Table Flag       1 Bit
//                              Color Resolution              3 Bits
//                              Sort Flag                     1 Bit
//                              Size of Global Color Table    3 Bits
        unsigned char background;   
        unsigned char aspect_ratio; 
    } giflsd;
    QFile file(path);
    if(!file.open(IO_ReadOnly)) {
        return -1;
    }
    QDataStream stream( &file );
    QIODevice * device = stream.device();
    QString str;
    Q_INT8 a;
    //char* magic[3];
    char signature[3];
    stream.readRawBytes(signature, 3 );
    if ( memcmp(signature, "GIF", 2) != 0 ) {
        return 0;
    }
    char version[3];
    stream.readRawBytes(version, 3 );
    stream.readRawBytes((char*)&giflsd, sizeof(giflsd));
    if ( giflsd.packed_fields & 0x80) { // has global color table
        // advance 3 x 2^(Size of Global Color Table+1) bytes
        int color_table_size = 1 << ((giflsd.packed_fields & 0x07)+1);
        device->at( device->at() + 3*color_table_size );
    }
    int images_count = 0;
    unsigned char id_1 = device->getch();
    unsigned char id_2;
    do {
        id_2 = device->getch();
        if ( id_1 == 0x21 ) {
          if ( id_2 == 0xf9 ) { // Graphic Control Extension
            images_count++;
          }
        }
        id_1 = id_2;
/*        if (block_id == 0x2c) {  // Image Separator
            // Image Descriptor
            struct GIFIDstruct {
                unsigned short left;
                unsigned short top;
                unsigned short width;
                unsigned short height;
                unsigned char packed_fields;
            } gifid;
            stream.readRawBytes((char*)&gifid, sizeof(gifid));
            if ( gifid.packed_fields & 0x08 ) { // has local color table
                int local_color_table_size = 1 << ((gifid.packed_fields & 0x07)+1);
                device->at( device->at() + 3*local_color_table_size );
            }
            unsigned char size;
            stream >> size;

        }
        else if (block_id == 0x21) {       // Extension Introducer
        }
        else if (block_id == 0x3b) {  // GIF Trailer
            break;
        }*/
    } while (! device->atEnd() );
    file.close();
    return images_count > 1;
}




