/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mycolorrequester.h"
#include "themeattribute.h"
#include "../config/themekreatorconfig.h"

#include <krestrictedline.h>
#include <kcolorbutton.h>
#include <kdialog.h>

#include <qvariant.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qcolor.h> 
#include <qlabel.h> 
#include <qtooltip.h>

MyColorRequester::MyColorRequester(QWidget *parent, const char *name, ThemeAttribute * attribute)
 : QWidget(parent, name)
{
    if ( !name )
      setName( "MyColorRequester" );
    //myColorRequesterLayout = new QGridLayout( this, 1, 3, 0,  KDialog::spacingHint(), "MyColorRequesterLayout");
    myColorRequesterLayout = new QHBoxLayout( this, 0,  KDialog::spacingHint(), "MyColorRequesterLayout");
    
    //lLabel = new QLabel("Name", this, "lLabel");
    //myColorRequesterLayout->addWidget( lLabel, 0, 0 );
    
    myColorRequesterLayout->addWidget( new QLabel("#", this), 0, 0 );

    leColor = new KRestrictedLine( this, "leColor" );
    leColor->setText("000000");
    currentColor = "000000";
    leColor->setMaxLength(6);
    leColor->setValidChars("0123456789ABCDEFabcdef");
    leColor->setSizePolicy( QSizePolicy( 
      (QSizePolicy::SizeType)2, 
      (QSizePolicy::SizeType)0, 
      0, 
      0, 
      leColor->sizePolicy().hasHeightForWidth() 
      )
    );
    leColor->setMinimumWidth(leColor->fontMetrics().width("XXXXXXX") );
    leColor->setMaximumWidth(leColor->fontMetrics().width("XXXXXXX") );

    myColorRequesterLayout->addWidget( leColor, 0, 1 );

    kColorButton = new KColorButton( this, "kColorButton" );
    kColorButton->setFixedSize(24, 24);
    kColorButton->setSizePolicy( QSizePolicy( 
      (QSizePolicy::SizeType)0, 
      (QSizePolicy::SizeType)0, 
      0, 
      0, 
      kColorButton->sizePolicy().hasHeightForWidth() 
      )
    );

    myColorRequesterLayout->addWidget( kColorButton, 0, 2 );
    
    //invalidColor = "#FF0000";
    //defaultColor = leColor->foregroundColor().name();
    
    connect( kColorButton, SIGNAL( changed(const QColor&) ), this, SLOT( updateColorText(const QColor&) ) );
    connect( kColorButton, SIGNAL( released() ), this, SLOT( updateColorButton() ) );
    connect( leColor, SIGNAL( lostFocus() ), this, SLOT( updateColorButton() ) );
    connect( leColor, SIGNAL( returnPressed() ), this, SLOT( updateColorButton() ) );


//     leColor->setFocusProxy(this);
//     this->setFocusPolicy(QWidget::StrongFocus);
//     connect( this, SIGNAL( focusInEvent ( QFocusEvent * ) ), this, SIGNAL( signalEnterFocus() ) );
//     connect( this, SIGNAL( focusOutEvent ( QFocusEvent * ) ), this, SIGNAL(signalLostFocus()));
    
    if ( attribute != 0) {
      setAttribute(attribute);    
    }    
}

MyColorRequester::~MyColorRequester()
{
}



void MyColorRequester::setAttribute(ThemeAttribute * a) {
  m_attribute = a;
  setText(m_attribute->get_value());
  //lLabel->setText(m_attribute->get_uiName());

  updateTooltip();

  updateColorButton( );
  
  connect(m_attribute, SIGNAL(signalValueChanged(const QString &)), this, SLOT(setText(const QString &)));
  connect(m_attribute, SIGNAL(signalInvalidChanged(bool )), this, SLOT(setInvalid(bool )));
}

void MyColorRequester::updateTooltip() {
  if ( m_attribute != 0) {
    if ( ThemeKreatorConfig::show_tooltips() ) {
      QToolTip::remove( leColor );
      QToolTip::add( leColor, m_attribute->get_description() );
    }
    else {
      QToolTip::remove( leColor );
    }
  }
}

void MyColorRequester::setInvalid(bool isInvalid) {
  if (isInvalid) {
    leColor->setPaletteForegroundColor ( Qt::red );
    //lLabel->setPaletteForegroundColor ( Qt::red );
  }
  else {
    leColor->setPaletteForegroundColor ( QPalette::foregroundRoleFromMode(leColor->backgroundMode()) );
    //lLabel->setPaletteForegroundColor ( QPalette::foregroundRoleFromMode(lLabel->backgroundMode()) );
  }
}

void MyColorRequester::updateColorText( const QColor & c )
{
  leColor->setText(c.name().mid(1,6).lower());
  m_attribute->set_value( c.name().mid(1,6).lower() );
}

void MyColorRequester::updateColorButton( )
{
  if (leColor->text().length() == 6) {
    kColorButton->setColor( QColor("#"+leColor->text().lower()) );
    currentColor = leColor->text();
  }
  else {
    kColorButton->setColor( QColor("#"+currentColor) );
    leColor->setText(currentColor);
  }
/*  else if (leColor->text().length() == 0) {
    // e.g. user deleted color completely
    kColorButton->setColor(QColor());
    leColor->setText("");
  }*/
}

QString MyColorRequester::text() 
{ 
  return leColor->text(); 
}

void MyColorRequester::setText(const QString & text) 
{ 
  if (leColor->text() != text) {
    leColor->setText(text); 
    updateColorButton();
  }
}






#include "mycolorrequester.moc"
