/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mykurlrequester.h"
#include "../config/themekreatorconfig.h"

#include <qlabel.h>
#include <qsize.h>
#include <qlayout.h>
#include <klocale.h>
#include <qtooltip.h>
#include <qimage.h>

#include <kaccel.h>
#include <kdebug.h>
#include <kdialog.h>
#include <kdirselectdialog.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klineedit.h>
#include <kurlcompletion.h>
#include <kmessagebox.h>
#include <kurldrag.h>
#include <ktempfile.h>
#include <krun.h>
#include <kurl.h>


/*class KURLDragPushButton : public KPushButton
{
public:
    KURLDragPushButton( QWidget *parent, const char *name=0 )
	: KPushButton( parent, name ) {
    	setDragEnabled( true );
    }
    ~KURLDragPushButton() {}

    void setURL( const KURL& url ) {
	m_urls.clear();
	m_urls.append( url );
    }

protected:
    virtual QDragObject *dragObject() {
	if ( m_urls.isEmpty() )
	    return 0L;

	QDragObject *drag = new KURLDrag( m_urls, this, "url drag" );
	return drag;
    }

private:
    KURL::List m_urls;

};*/


MyKURLRequester::MyKURLRequester(QWidget *parent, const char *name, QString imagefilter, ThemeAttributeImage * attribute)
 : QHBox(parent, name)
{
    if ( !name )
      setName( "MyKURLRequester" );
    this->m_imagefilter = imagefilter;
    edit = 0;
    myFileDialog    = 0;
    edit = new KLineEdit( this, "line edit" );
    btnEdit = new /*KURLDragPushButton*/ KPushButton( this, "edit button");
    QIconSet iconSetEdit = SmallIconSet(QString::fromLatin1("edit"));
    QPixmap pixMapEdit = iconSetEdit.pixmap( QIconSet::Small, QIconSet::Normal );
    btnEdit->setIconSet( iconSetEdit );
    btnEdit->setFixedSize( pixMapEdit.width()+8, pixMapEdit.height()+8 );
    QToolTip::add(btnEdit, i18n("Edit image"));
    btnEdit->setEnabled(false);
    btnOpen = new /*KURLDragPushButton*/ KPushButton( this, "kfile button");
    QIconSet iconSetOpen = SmallIconSet(QString::fromLatin1("fileopen"));
    QPixmap pixMapOpen = iconSetOpen.pixmap( QIconSet::Small, QIconSet::Normal );
    btnOpen->setIconSet( iconSetOpen );
    btnOpen->setFixedSize( pixMapOpen.width()+8, pixMapOpen.height()+8 );
    QToolTip::add(btnOpen, i18n("Open file dialog"));

    //connect( btnOpen, SIGNAL( pressed() ), SLOT( slotUpdateButton() ));
    setSpacing( KDialog::spacingHint() );
    setFocusProxy( edit );
    //connect( edit, SIGNAL( textChanged( const QString& )), this, SIGNAL( textChanged( const QString& )));
    //connect( edit, SIGNAL( returnPressed() ), this, SIGNAL( returnPressed() ));
    //connect( edit, SIGNAL( returnPressed( const QString& ) ), this, SIGNAL( returnPressed( const QString& ) ));
    connect( btnOpen, SIGNAL( clicked() ), this, SLOT( slotOpenDialog() ));
    connect( btnEdit, SIGNAL( clicked() ), this, SLOT( slotEdit() ));
    
    connect( edit, SIGNAL( lostFocus() ), this, SLOT( slotUpdateURL() ) );
    connect( edit, SIGNAL( returnPressed() ), this, SLOT( slotUpdateURL() ) );
    
    myCompletion = new KURLCompletion();
    edit->setCompletionObject( myCompletion );
   
    KAccel *accel = new KAccel( this );
    accel->insert( KStdAccel::Open, this, SLOT( slotOpenDialog() ));
    accel->readSettings();

    m_attribute = 0;
    if ( attribute != 0) {
      setAttribute(attribute);    
    }
    resized_image = 0;
}

void MyKURLRequester::slotUpdateURL() {
  if (m_attribute != 0) {
      if ( m_attribute->get_value() != edit->text()  ) {
        m_attribute->set_value( edit->text(),  true);
      }
  }
}

void MyKURLRequester::setInvalid(bool isInvalid) {
/*  if (isInvalid) {
    leColor->setPaletteForegroundColor ( Qt::red );
    //lLabel->setPaletteForegroundColor ( Qt::red );
  }
  else {
    leColor->setPaletteForegroundColor ( QPalette::foregroundRoleFromMode(leColor->backgroundMode()) );
    //lLabel->setPaletteForegroundColor ( QPalette::foregroundRoleFromMode(lLabel->backgroundMode()) );
  }*/
}

MyKURLRequester::~MyKURLRequester()
{
  if ( resized_image != 0) {
    delete resized_image;
  }
  delete myCompletion;
}


void MyKURLRequester::setAttribute(ThemeAttributeImage * a) {
  m_attribute = a;
  setURL(m_attribute->get_value());
  //lLabel->setText(m_attribute->get_uiName() + sizeAsText(m_attribute->get_maxSize()) );
/*  if ( myFileDialog != 0) {
    myFileDialog->setCaption( i18n("Select image for ") + m_attribute->get_uiName() );
  }*/

  updateTooltip();

  /*connect(this, SIGNAL(urlChanged(const QString &)), m_attribute, SLOT(set_value(const QString &)));
  connect(urlRequester, SIGNAL(textChanged (const QString &)), m_attribute, SLOT(set_value(const QString &)));
  */
  connect(m_attribute, SIGNAL(signalValueChanged(const QString &)), this, SLOT(setURL(const QString &)));
  connect(m_attribute, SIGNAL(signalValueChanged(const QString &)), this, SLOT(slotToggleEditButton()));
  
  connect(m_attribute, SIGNAL(signalInvalidChanged(bool )), this, SLOT(setInvalid(bool )));
}

void MyKURLRequester::updateTooltip() {
  if (m_attribute != 0) {
    if ( ThemeKreatorConfig::show_tooltips() ) {
      QToolTip::remove( edit );
      QToolTip::add( edit, m_attribute->get_description() );
    }
    else {
      QToolTip::remove( edit );
    }
  }
}

void MyKURLRequester::setURL( const QString& url )
{
  if (edit->text() != url) {
    edit->setText( url );
    //updateColorButton();
   // emit urlChanged(url);
  }
}

QString MyKURLRequester::url() const
{
    return edit->text();
}

void MyKURLRequester::slotOpenDialog()
{
    KURL newurl;    

    KFileDialog *dlg = fileDialog();

//     if ( ! m_attribute->get_value().isEmpty() ) {
//       dlg->setURL( m_attribute->get_directory() + m_attribute->get_value() );
//       dlg->setSelection( m_attribute->get_value() );
//     }

    if ( dlg->exec() != QDialog::Accepted )
    {
      return;
    }
                
    newurl = dlg->selectedURL();
    
    if (m_attribute != 0) {
      QImage img = QImage(newurl.path());
      int result;
      if (img.width() > m_attribute->get_maxWidth() || img.height() > m_attribute->get_maxHeight()) {
        result = KMessageBox::questionYesNoCancel( this, i18n("The image you've selected is too large for this element.\n\n Do you want to resize this element automatically to the appropiate size? Press Cancel to abort this action completely."), i18n("Image too Large for Element") );
      } 
      if (result == KMessageBox::Cancel) { 
        return;
      }
      else if ( result == KMessageBox::Yes ) {
        if ( resized_image != 0) {
          delete resized_image;
          resized_image = 0;
        }
        resized_image = new KTempFile(QString::null, "."+QString(img.imageFormat(newurl.path())).lower() );
        resized_image->setAutoDelete(true);
        img.smoothScale(m_attribute->get_maxWidth(), m_attribute->get_maxHeight(),  QImage::ScaleFree).save(resized_image->name(), img.imageFormat(newurl.path()));

        KURL tmpurl = KURL(resized_image->name());
        m_attribute->set_fullpath( tmpurl.filename(), tmpurl.directory(false),  true);
        
      } else {
        m_attribute->set_fullpath( newurl.filename(), newurl.directory(false),  true);
      }
    }
}

void MyKURLRequester::slotEdit()
{
  QString program = ThemeKreatorConfig::pixelEditor();
  if ( program.isEmpty() ) {
    KMessageBox::sorry(this, i18n("To use the inplace editing feature, you must enter your preferred image editor in the settings dialog of ThemeKreator."));
    return;
  }
  QString command = program + " " + m_attribute->get_directory()+m_attribute->get_value();
  KRun::runCommand(command, program, "edit");
}

void MyKURLRequester::setFilter(const QString &filter)
{
    if (myFileDialog)
       myFileDialog->setFilter( filter );
}

KFileDialog * MyKURLRequester::fileDialog() const
{
  if ( !myFileDialog ) {
    QWidget *p = parentWidget();
    //if ( m_attribute->get_value().isEmpty() ) {
      myFileDialog = new KFileDialog( ":image", QString::null, p, "file dialog", true );
    //}
    //else {
    //  myFileDialog = new KFileDialog( m_attribute->get_directory() + m_attribute->get_value(), QString::null, p, "file dialog", true );
    //}
    myFileDialog->setCaption( i18n("Select image for ") + m_attribute->get_uiName() );
    QString filter = "";
    if (m_imagefilter != QString::null) {
      filter += m_imagefilter + "|"+i18n("All image formats supported by the phone")+"\n";
    }
    filter += "*.*|"+i18n("All files");
    myFileDialog->setFilter(filter);
    myFileDialog->setMode(1);
  }
  return myFileDialog;
}

void MyKURLRequester::slotToggleEditButton() {
  if (m_attribute != 0) {
    if ( m_attribute->get_value().isEmpty() ) {
      btnEdit->setEnabled(false);
    }
    else {
      btnEdit->setEnabled(true);
    }
  }
  else {
    btnEdit->setEnabled(false);
  }
}


/*QString MyKURLRequester::sizeAsText(QSize s) {
  if (s.isValid())
   return QString( " (%1 x %2)").arg( s.width() ).arg( s.height() );
  else
    return "";
}*/

#include "mykurlrequester.moc"
