/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mylineedit.h"
#include "../config/themekreatorconfig.h"
#include "themeattribute.h"

#include <qlabel.h>
#include <qsize.h>
#include <qlayout.h>
#include <qtooltip.h>

#include <klocale.h>
#include <kdialog.h>


MyLineEdit::MyLineEdit(QWidget *parent, const char *name, ThemeAttribute * attribute)
 : KLineEdit(parent, name)
{
  if ( !name ) {
    setName( "MyLineEdit" );
  }
  m_attribute = 0;
  
  connect( this, SIGNAL( lostFocus() ), this, SLOT( slotUpdateText() ) );
  connect( this, SIGNAL( returnPressed() ), this, SLOT( slotUpdateText() ) );  
  
  if ( attribute != 0) {
      setAttribute(attribute);    
  }    
}

MyLineEdit::~MyLineEdit()
{
}

void MyLineEdit::setAttribute(ThemeAttribute * a) {
  m_attribute = a;
  KLineEdit::setText(m_attribute->get_value());
  //lLabel->setText(m_attribute->get_uiName());

  updateTooltip();
  
  connect(m_attribute, SIGNAL(signalValueChanged(const QString &)), this, SLOT(setText(const QString &)));
  //connect(m_attribute, SIGNAL(signalInvalidChanged(bool )), this, SLOT(setInvalid(bool )));
}

void MyLineEdit::updateTooltip() {
  if (m_attribute != 0) {
    if ( ThemeKreatorConfig::show_tooltips() ) {
      QToolTip::remove( this );
      QToolTip::add( this, m_attribute->get_description() );
    }
  }
}

void MyLineEdit::slotUpdateText() {
  if (m_attribute != 0) {
      m_attribute->set_value( text(),  true);
  }
}


void MyLineEdit::setText(const QString & text) 
{ 
  if (this->text() != text) {
    KLineEdit::setText(text);
    m_attribute->set_value( text );
  }
}

#include "mylineedit.moc"
