/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "phonepreview.h"

#include "previewitem.h"
#include "previewitemimage.h"

#include <qpainter.h>
#include <qptrlist.h>
#include <qpixmap.h>
#include <qtooltip.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qpaintdevice.h>


PhonePreview::PhonePreview(QWidget *parent, const char *name, PreviewDataModel *model)
 : QWidget(parent, name)
{
  data_model = 0;
  if (model != 0) {
    setPreviewDataModel(model);
  }
  setBackgroundMode(NoBackground);
}


PhonePreview::~PhonePreview()
{
}

void PhonePreview::setPreviewDataModel(PreviewDataModel *model) {
  this->data_model = model;
  connect(data_model, SIGNAL( signalValueChanged () ), this, SLOT( update () ));
  setFixedSize(data_model->get_preview_width(), data_model->get_preview_height());
  
  QPtrList<PreviewItem> *list = (QPtrList<PreviewItem> *) data_model;
  PreviewItem *item;
  for ( item = list->first(); item; item = list->next() ) {
    QToolTip::remove( this, QRect(item->getLeft(), item->getTop(), item->getWidth(), item->getHeight()) );
    QToolTip::add( this, QRect(item->getLeft(), item->getTop(), item->getWidth(), item->getHeight()), item->getToolTip() );
  }
  update ();
}


void PhonePreview::paintEvent( QPaintEvent * event )
{
  QPixmap pm(size());
  QPainter p;
  p.begin(&pm);//, this);
  p.fillRect( 0, 0, this->width(), this->height(), Qt::black );
  if ( data_model != 0) {
    QPtrList<PreviewItem> *list = (QPtrList<PreviewItem> *) data_model;
    PreviewItem *item;
    for ( item = list->first(); item; item = list->next() ) {
        item->draw(&p);
    }
  }
  p.end();
  bitBlt(this, 0, 0, &pm);
}

void PhonePreview::savePreview( QString filename ) {
  QPainter *p = new QPainter(); 
  QPixmap *img = new QPixmap(this->width(), this->height());
  p->begin( img );                  // start painting the widget
  p->fillRect( 0, 0, this->width(), this->height(), Qt::black );
  
  if ( data_model != 0) {
    QPtrList<PreviewItem> *list = (QPtrList<PreviewItem> *) data_model;
    PreviewItem *item;
    for ( item = list->first(); item; item = list->next() ) {
      item->draw(p);
    }
  }
  p->end();                          // painting done
  img->save(filename+".png", "PNG", -1);
}

void PhonePreview::movieUpdated(const QRect& area) {
  QRect r = childrenRect();//contentsRect();
  // Only redraw the changed area of the frame
  update(area.x()+r.x(), area.y()+r.x(), area.width(), area.height() );
}

QWidget *PhonePreview::previewWidget(QWidget *parent, const char *name, PreviewDataModel *model) {
  QFrame * frame = new QFrame(parent);
  frame->setFrameShadow(QFrame::Sunken);
  frame->setFrameShape(QFrame::LineEditPanel);
  frame->setPaletteBackgroundColor( Qt::black );
  frame->setMargin(0);
  QGridLayout * frameLayout = new QGridLayout(frame, 1, 1, 8);
  PhonePreview *p = new PhonePreview(frame, name, model);
  
  QPtrList<PreviewItem> *list = (QPtrList<PreviewItem> *) model;
  PreviewItem *item;
  for ( item = list->first(); item; item = list->next() ) {
    if (item->getType() == "image") {
      PreviewItemImage *image_item = (PreviewItemImage *)item;
      connect(image_item, SIGNAL(redraw(const QRect&)), p, SLOT(movieUpdated(const QRect&)));
    }
  }


  
  frameLayout->addWidget(p, 0, 0);
  return frame;
}


#include "phonepreview.moc"
