/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "selectmodeldialog.h"

#include "selectmodelwidget.h"
#include "phonemodelloader.h"
#include "selectmodeliconview.h"

#include <qlayout.h>

#include <klocale.h>
#include <qlabel.h>



SelectModelDialog::SelectModelDialog(const QString &version, QWidget * parent, const char * name)
 : KDialogBase(parent, 
      "selectModelDialog",
      true, 
      i18n("Select Phone Model"),
      KDialogBase::Ok|KDialogBase::Cancel, 
      KDialogBase::Ok,
      false)
{
  m_sel_model = QString::null;
  this->version = version;
  init();
  this->enableButtonOK(false); 
}

void SelectModelDialog::init() {
  PhoneModelLoader * loader = new PhoneModelLoader(this);
  PhoneModelGroup * group = loader->getGroup(version);
  if ( group == 0 ) {
    this->deleteLater();
    return;
  }
  QWidget *page = new QWidget( this );
  setMainWidget(page);
  QVBoxLayout * layout = new QVBoxLayout(page, KDialog::marginHint(), KDialog::spacingHint());

  QLabel *label = new QLabel(page);
  label->setText(i18n("The exact model, for which this theme was designed, could not be determined.\nPlease select a device from the list below."));
  layout->addWidget(label);
  icon_view = new SelectModelIconView(loader, group->getName(), page);
  layout->addWidget(icon_view);
  
  connect( icon_view, SIGNAL( clicked ( QIconViewItem * ) ), this, SLOT( currentChanged( QIconViewItem * ) ) );
  connect( icon_view, SIGNAL( clicked ( QIconViewItem * ) ), this, SLOT( selectedModelChanged( QIconViewItem * ) ) );
  connect( icon_view, SIGNAL( doubleClicked( QIconViewItem * ) ), this, SLOT( chosen(QIconViewItem *) ) );
  
  this->adjustSize();
  this->setInitialSize(QSize(600, 440));
}

// void SelectModelDialog::selectedModelChanged (QIconViewItem * item) {
//   m_sel_model = item->text();
// }

SelectModelDialog::~SelectModelDialog()
{
}

QString SelectModelDialog::getSelectedModel() {
  //return selectModelWidget->getSelectedItemText();
  return m_sel_model;
}


void SelectModelDialog::selectedModelChanged (QIconViewItem * item) {
  if (item) {
    this->enableButtonOK( true );
    m_sel_model = item->text();
  }
  else {
    this->enableButtonOK( false );
    m_sel_model = QString::null;
  }
}

void SelectModelDialog::currentChanged( QIconViewItem * item)
{
}

void SelectModelDialog::chosen(QIconViewItem * item)
{
  this->accept();
}


#include "selectmodeldialog.moc"
