/***************************************************************************
 *   Copyright (C) 2004 by Matthias Wetzka                                 *
 *   matthias.wetzka@gmx.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "selectmodelwidget.h"

#include "phonemodelloader.h"
#include "selectmodeliconview.h"

#include <qstrlist.h>
#include <qpixmap.h>
#include <qvgroupbox.h>
#include <qlayout.h>
#include <qptrlist.h>

#include <klocale.h>
#include <kconfig.h>
#include <kjanuswidget.h>
#include <kstandarddirs.h>
#include <ktextedit.h>
#include <kdialogbase.h>


SelectModelWidget::SelectModelWidget(QWidget *parent, const char *name)
 :  QWidget( parent, name )
{
  m_jwidget = new KJanusWidget(this, "kjanuswidget", KJanusWidget::IconList);
  QGridLayout * layout = new QGridLayout( this, 1, 1, KDialogBase::marginHint(), KDialogBase::spacingHint());
  layout->addWidget(m_jwidget,0,0);

  m_descriptiongroup = new QVGroupBox(i18n("Selected Phone Model"), this, "description");
  m_descriptiongroup->setInsideMargin ( 3 );
  m_descriptiongroup->setInsideSpacing ( 3 );
  layout->addWidget(m_descriptiongroup, 1, 0 );

  layout->setRowStretch( 0, 1 );
  layout->setRowStretch( 1, 0 );

  m_textedit = new KTextEdit( m_descriptiongroup );
  m_textedit->setReadOnly(1);
  m_textedit->setText(i18n("Please select a phone model."));
  m_textedit->setLineWidth(0);
  m_textedit->setMaximumHeight(60);

  // item which will be selected initially
  m_selected = 0;

  // count the templates inserted
  int entriesnumber = 0;
  int defaultTemplateGroup = -1;

  PhoneModelLoader * loader = new PhoneModelLoader(this);
  QPtrList<PhoneModelGroup> * groups = loader->getGroups();
  PhoneModelGroup * group;
  for ( group = groups->first(); group; group = groups->next() ) {
    QFrame * frame = m_jwidget->addPage(i18n(group->getName()), i18n("Available %1 phone models:").arg(group->getName()),  group->getIcon());

    SelectModelIconView * icon_view = new SelectModelIconView(loader, group->getName(), frame);
    QGridLayout * layout = new QGridLayout(frame);
    layout->addWidget(icon_view,0,0);
    connect( icon_view, SIGNAL( clicked ( QIconViewItem * ) ), this, SLOT( currentChanged( QIconViewItem * ) ) );
    connect( icon_view, SIGNAL( clicked ( QIconViewItem * ) ), parent, SLOT( selectedModelChanged( QIconViewItem * ) ) );
    connect( icon_view, SIGNAL( doubleClicked( QIconViewItem * ) ), parent, SLOT( chosen(QIconViewItem *) ) );
    
    /*QIconView * icon_view = new QIconView(frame);
    
    QGridLayout * layout = new QGridLayout(frame);
    layout->addWidget(icon_view,0,0);
    connect( icon_view, SIGNAL( clicked ( QIconViewItem * ) ), this, SLOT( currentChanged( QIconViewItem * ) ) );
    connect( icon_view, SIGNAL( clicked ( QIconViewItem * ) ), parent, SLOT( selectedModelChanged( QIconViewItem * ) ) );
    connect( icon_view, SIGNAL( doubleClicked( QIconViewItem * ) ), this, SLOT( chosen(QIconViewItem *) ) );
    icon_view->setBackgroundColor( colorGroup().base() );
    icon_view->setResizeMode(QIconView::Adjust);
    icon_view->setWordWrapIconText( true );
    icon_view->show();
    QPtrList<PhoneModelItem> * models = loader->getModels(group->getName());
    PhoneModelItem * model;
    for ( model = models->first(); model; model = models->next() ) {
      ModelIconViewItem * item = new ModelIconViewItem(icon_view, model->getName(), model->getIcon(), model->getDescription(), "none");
      //item->setKey(model);
      item->setDragEnabled(false);
      item->setDropEnabled(false);
    }*/
  }
}


QString SelectModelWidget::getSelectedItemText() {
//  return firstItem()->text();
  //this->selectedItem()->text();
  //firstItem()->text ();
  if ( m_selected )
    return m_selected->text();
  else
    return QString::null;
}

SelectModelWidget::~SelectModelWidget()
{
}


QString SelectModelWidget::descriptionText(const QString &name, const QString &description)
{
  QString descrText(i18n("Name:"));
  descrText += " " + name;
  descrText += "\n";
  descrText += i18n("Description:");
  if (description.isEmpty())
    descrText += " " + i18n("No description available");
  else
    descrText += " " + description;
  return descrText;
}


void SelectModelWidget::currentChanged( QIconViewItem * item)
{
  if (m_selected != 0 ) {
    m_selected->iconView()->setSelected( m_selected, false );
  }
  if (item) {
    //if (m_selected == item) return;
    QIconView* icon_view =  item->iconView();
    // set text in the textarea
    m_textedit->setText( descriptionText(item->text(), ((SelectModelIconViewItem *) item)->getDescr()) );
    m_selected = item;
  }
  else {
    m_textedit->setText( i18n("Please select a phone model.") );
    m_selected = 0;
  }
}




#include "selectmodelwidget.moc"
